% FDMwideAngleFlat.m : Computes the field using a wide-angle approximation,
%                      the Claerbout approximation.

function [uValues] = FDMwideAngleFlat(zFieldInit, ...
    zVect,xVect,maxHeigthInterestZIndex,frequency,g)

c = 3*10^8;
lambda = c/frequency;
k = 2*pi/lambda;
numZpoints = length(zVect);
numXpoints = length(xVect);
Hindex = maxHeigthInterestZIndex;
xVect= verticalVector(xVect);
zVect = verticalVector(zVect);

deltaZ = zVect(2,1)-zVect(1,1);
deltaZvect = zeros(numZpoints,1);
deltaZvect(1:numZpoints-1,1)= zVect(2:numZpoints,1)-zVect(1:numZpoints-1);
deltaZvect(numZpoints,1) = deltaZvect(numZpoints-1,1);

deltaX = xVect(2,1)-xVect(1,1);
deltaXvect = zeros(numXpoints,1);
deltaXvect(1:numXpoints-1,1)= xVect(2:numXpoints,1)-xVect(1:numXpoints-1);
deltaXvect(numXpoints,1) = deltaXvect(numXpoints-1,1);


% Hanning window for absorption layer:
indiceValues = (verticalVector([0:1:numZpoints-Hindex]))./(numZpoints-Hindex);
%absorptionLayer = (1 +cos(pi.*indiceValues))./2;

% Creating the absortion layer using a damping function, geometrical
% damping
gamma0 = 0.1;
%g = 0.01;
absorptionLayer = gamma0.*(g.^(indiceValues./deltaZvect(1,1)));

indexRefraction = ones(numZpoints,1);
indexRefraction(Hindex:numZpoints,1) = indexRefraction(Hindex:numZpoints,1) ...
    + 1j.*absorptionLayer;

% Finding the ''grid'' of a- and b-values:



% Vector a- and b-grid:
%bGrid = zeros(numZpoints,1);
aGrid = zeros(numZpoints,1);
%bGrid(:,1) = 1j.*4.*k.*(deltaZvect.^2)./deltaX;
bGrid = ones(numZpoints,1).*((4*(k.^2).*deltaZ).^2)./(1 + 1j.*k.*deltaZ);
bGrid(1,1) = 1;
bGrid(numZpoints,1) = 1;
cGrid = (1 - 1j.*k.*deltaX)./(1 + 1j.*k.*deltaX);
aGrid(:,1) = k.^2.*(indexRefraction.^2 -1).*(deltaZ.^2);
alphaValues = -2 + cGrid + aGrid;
alphaValues(1,1) =1;
alphaValues(numZpoints,1) =1;


% Calculate the u-values for successive x-values:
uGrid = zeros(numZpoints,numXpoints);
uGrid(:,1)= zFieldInit;
onesVect = ones(numZpoints-1,1);

%disp('Create Vmat')
diagVmat = 2 + bGrid- aGrid(:,1);
diagVmat(1,1) = 1;
diagVmat(numZpoints,1) = 1;
VmMat = (diag(diagVmat)+ (-1.*diag(onesVect,-1))+(-1.*diag(onesVect,1))); 

%disp('Create Amat')
 diagAmat = -2 + cGrid(1,1)+ aGrid(:,1);
 diagAmat(1,1) = 1;
 diagAmat(numZpoints,1) = 1;
 AmMat = (diag(diagAmat)+diag(onesVect,-1) + diag(onesVect,1));


% Crank-Nicolson scheme to solve the problem:
vVect = VmMat*double(zFieldInit);
tau = zeros(numZpoints,1);
for x = 2:numXpoints
%     tau(1) = vVect(1)/alphaValues(1,1);
%     for z = 2: numZpoints
%         tau(z) = double((vVect(z,1)-tau(z-1,1))./alphaValues(z,1));
%     end
%   
%     uGrid(numZpoints,x) = tau(numZpoints);
%     for z = (numZpoints-1):-1:1
%     
%         
%         uGrid(z,x) = double(tau(z) - uGrid(z+1,x)/alphaValues(z));
%     end
     uGrid(:,x) = linsolve(AmMat,vVect);
    vVect = VmMat*uGrid(:,x);
    x   
end

uValues = uGrid;
uValues(abs(uValues)<10^-3) = 10^-3;
uValues(abs(uValues)>1) = 1;
%uValues(abs(uValues)>10^-1) = 10^-1;

end
        
    




