% Luton2.m: Script simulating and comparing  the electric field over the 
%           Luton airport.
clear all

% Setting the parameters:
theta0 = 0;
beta = pi/18; 
A = 1;
frequency = 110*10^6;

deltaX = 1;
maxX = 3000;
numPtsAbsoptionLayer = 150;

deltaZ = 1;
antHeight = 3;
deltaXvect = [1];

counter = 0;
doubleCounter = 0;
maxHinterestHeight = 10;
numElts = 2;
simulationType = cell(numElts,1);

% Get the terrain profile:
xColumn = 1;
zColumn = 3;
fileName ='\IndraSource\luton\Runway-profile_LOC26.xls'; %
%'IndraSource/luton/Runway-profile_LOC26.xls';

% Importing, interpolating, and shifting the surface height:
[Xn,Zn]=importParametersFromFile(fileName,xColumn,zColumn);

[xVect,zSurfaceVect] = interpolate(Xn,Zn,frequency,...
    'curve',deltaX);
[zSurfaceNorm,truncationValue]=normalizeSurface(zSurfaceVect);

antHeight = antHeight + zSurfaceNorm(1);

maxHinterest = 350 +antHeight;

% Plot the surface:
surfacePlot = figure();
plot(xVect,zSurfaceNorm)
xlabel('Distance [m]');
ylabel('Surface height [m]');
plotwidth = 560;
plotheight = 200;
set(surfacePlot, 'Position', [500 100 plotwidth plotheight]);
legend('Runway surface profile','Location','SouthWest');
titleFig = ['Luton_results/Runway_Luton.png'];
saveas(surfacePlot,titleFig,'png');
titleFig = titleFig(1:(length(titleFig)-4));
titleFig = horzcat(titleFig,'.pdf');
%print (fig, '-dpdf', titleFig);
save2pdf(titleFig);

% Creating initial field:
[zVectFDM,HindexFDM] =createZvectAbsorptionLayer2(...
    maxHinterest,deltaZ,numPtsAbsoptionLayer);
initialFieldFDM = createInitialField(antHeight,theta0,beta,...
    zVectFDM,A, frequency,'gaussian1');
numZpoints = length(zVectFDM);

% Calculating field:
tic
uValuesSplitStep = SSAirregularTerrainAbsoptionLayer(...
    initialFieldFDM,zVectFDM,...
    xVect,zSurfaceNorm,HindexFDM,frequency,...
    numPtsAbsoptionLayer,antHeight);
toc
tic
uValuesFDM = FDMirregularTerrainAbsorptionLayer ...
    (initialFieldFDM,zVectFDM,xVect,zSurfaceNorm,...
    HindexFDM,frequency,numPtsAbsoptionLayer);
toc
deltaZstr = num2str(deltaZ);
yText = strcat('Height above the lowest point [m]');

SurfEndHeight = ceil(zSurfaceNorm(length(zSurfaceNorm)));
fieldVect =[...
    ceil(SurfEndHeight/deltaZ):1:floor(length(zVectFDM)/deltaZ)-1]+1;

% Extracting the field values for vertical comparison:
eFieldSSA = zeros(1,length(fieldVect));
eFieldFDM = zeros(1,length(fieldVect));
for i = 1:length(fieldVect)
    eFieldSSA(1,i) = uValuesSplitStep(fieldVect(i),...
        length(xVect));
    eFieldFDM(1,i) = uValuesFDM(fieldVect(i),...
        length(xVect));
end

simulationType{1,1} = ['SSA'];
simulationType{2,1} = ['FDM'];
tx = antHeight;
rx = antHeight;

eFieldTot = eFieldSSA;
eFieldTot = vertcat(eFieldTot,eFieldFDM);


% Plot SSA field:
fig2 = figure('visible','off');
part1Title = ['Split-Step Algorithm - flat surface'];
titleVal2 = {part1Title};

uValuesAux = uValuesSplitStep;
uValuesAux(abs(uValuesAux)<10^-8) = 10^-8;
contourf(xVect,zVectFDM,10.*log10(abs(uValuesAux.^2)),50)
hold on
contour(xVect,zVectFDM,10.*log10(abs(uValuesAux.^2)),50)
%title(titleVal2)
xlabel('Distance [m]');
ylabel(yText);
grid on
titleFig = ['Luton_results/SSA_flat_along_surface_Luton.png'];
saveas(fig2,titleFig,'png');

% Plot FDM field:
fig = figure('visible','off');
part1Title = ['Finite-Difference Method - flat surface'];
titleVal2 = {part1Title};

uValuesAux = uValuesFDM;
uValuesAux(abs(uValuesAux)<10^-8) = 10^-8;
contourf(xVect,zVectFDM,10.*log10(abs(uValuesAux.^2)),50)
hold on
contour(xVect,zVectFDM,10.*log10(abs(uValuesAux.^2)),50)
%title(titleVal2)
xlabel('Distance [m]');
ylabel(yText);
grid on
titleFig = ['Luton_results/FDM_flat_along_surface_Luton.png'];
saveas(fig,titleFig,'png');

% Comparing with results from Indra, vertical comparison:
tx =antHeight;
rx = antHeight;
rx_xArr = ones(length(zVectFDM),1).*length(xVect);
rx_zArr = fieldVect; % zVectFDM;
part1Title = ' ';%['SSA and FDM - flat surface - along the surface'];
plotTitle = {part1Title};
titleFig = ['Luton_results/SSA_FDM_flat_along_surface_Luton.png'];
filename = 'IndraWedge2_results/LPDA-u-kile-2_E.xls';

pathLossWedge_Indra(Xn,Zn,deltaX,A,tx,rx_xArr,rx_zArr,eFieldTot,...
    antHeight,beta,frequency,simulationType, plotTitle,titleFig,' ')

% Comparing with results from a flat surface
antHeight2 = antHeight - zSurfaceNorm(1);
[zVectFDM,HindexFDM] =createZvectAbsorptionLayer2(...
    maxHinterest,deltaZ,numPtsAbsoptionLayer);
initialFieldFDM_flat = createInitialField(antHeight2,theta0,beta,...
    zVectFDM,A, frequency,'gaussian1');

% Calculating field over flat surface:
tic
uValuesSSA2 =splitStepAlgorithmAbsorptionLayer(...
    initialFieldFDM_flat,zVectFDM,xVect, ...
    HindexFDM,frequency,numPtsAbsoptionLayer);
toc

tic
[uValuesFDM2,maxEigVal]=FDMAbsorptionLayerNumEfficient2(...
    initialFieldFDM_flat,...
    zVectFDM,xVect,HindexFDM,frequency,numPtsAbsoptionLayer);
toc

% Extracting field values for vertical comparison:
eFieldSSA2 = zeros(1,length(fieldVect));
eFieldFDM2 = zeros(1,length(fieldVect));
for i = 1:length(fieldVect)
    eFieldSSA2(1,i) = uValuesSSA2(fieldVect(i),...
        length(xVect));
    eFieldFDM2(1,i) = uValuesFDM2(fieldVect(i),...
        length(xVect));
end


eFieldTot = vertcat(eFieldTot,eFieldSSA2);
eFieldTot = vertcat(eFieldTot,eFieldFDM2);

rx_xArr2 = ones(length(zVectFDM),1).*length(xVect);
rx_zArr2 =  fieldVect; 

titleFig2 = ['Luton_results/',...
    'SSA_FDM_pathloss_vertical_Luton.png'];
filename2 = 'IndraWedge_results/LPDA-u-kile_E.xls';
simulationType2 = cell(4,1);
simulationType2{1,1} = ['SSA - runway'];
simulationType2{2,1} = ['FDM - runway'];
simulationType2{3,1} = ['SSA - flat'];
simulationType2{4,1} = ['FDM - flat'];
compareHeight = 200;

compareHeight_Hviid = 350;

antHeight2 = antHeight - zSurfaceNorm(1);
% Vertical comparison:
pathLossWedge_Hviid(antHeight2,rx_xArr2,rx_zArr2,eFieldTot,...
    frequency,simulationType2, ' ',titleFig2,' ',compareHeight_Hviid)


% Compare along the surface at constant heigth of 40 m above the lowest
% point:

height = 40;
startIndex = 101; 
eSSArunway = (uValuesSplitStep(height,startIndex:length(xVect)));
eFieldAlongTot  =  eSSArunway;
eFDMrunway = (uValuesFDM(height,startIndex:length(xVect)));
eFieldAlongTot = vertcat(eFieldAlongTot,eFDMrunway);

eSSAflat = (uValuesSSA2(height,startIndex:length(xVect)));
eFieldAlongTot = vertcat(eFieldAlongTot,eSSAflat);
eFDMflat = (uValuesFDM2(height,startIndex:length(xVect)));
eFieldAlongTot = vertcat(eFieldAlongTot,eFDMflat);

rx_xArr3 = xVect(startIndex:length(xVect));
rx_zArr3 =  ones(length(rx_xArr3),1).*height;
titleFig3 = ['Luton_results/',...
    'SSA_FDM_pathloss_horizontal_Luton.png'];

pathLossWedge_Hviid(antHeight2,rx_zArr3,rx_xArr3,eFieldAlongTot,...
    frequency,simulationType2, ' ',titleFig3,' ',0)
titleFig4 = ['Luton_results/',...
    'SSA_FDM_pathloss_horizontal2_Luton.png'];

pathLossIndra_alongX(rx_xArr3,rx_zArr3,eFieldAlongTot,...
    frequency,simulationType2,' ',titleFig4,' ')


% Comparing fields along the surface at constant height above the surface
height2 = 10;
startIndex2 = 101;
fieldVect2 = [ceil(startIndex2/deltaX):length(xVect)/deltaX];
eSSArunway2 = zeros(length(fieldVect2),1);
eFDMrunway2 = zeros(length(fieldVect2),1);
for i = 1:length(fieldVect2)
    %a = zSurfaceNorm(i);
    eSSArunway2(i,1) = uValuesSplitStep(round(zSurfaceNorm(i)+height), ...
        fieldVect2(i));
    eFDMrunway2(i,1) = uValuesFDM(round(zSurfaceNorm(i)+height), ...
        fieldVect2(i));
end

eFieldAlongTot2  =  eSSArunway2';
eFieldAlongTot2 = vertcat(eFieldAlongTot2,eFDMrunway2');
eSSAflat2 = (uValuesSSA2(height,startIndex2:length(xVect)));
eFieldAlongTot2 = vertcat(eFieldAlongTot2,eSSAflat2);
eFDMflat2 = (uValuesFDM2(height,startIndex2:length(xVect)));
eFieldAlongTot2 = vertcat(eFieldAlongTot2,eFDMflat2);

eFieldAlongTot2Inv = eFieldAlongTot2;

for i =0:3
    eFieldAlongTot2Inv(i+1,:) = eFieldAlongTot2(4-i,:);
end

titleFig5 = ['Luton_results/',...
    'SSA_FDM_pathloss_horizontal_cst_diff_surface_Luton.png'];
rx_xArr4 = xVect(startIndex2:length(xVect));
rx_zArr4 =  ones(length(rx_xArr4),1).*height;

simulationTypeInv = cell(4,1);
simulationTypeInv{4,1} = ['SSA - runway'];
simulationTypeInv{3,1} = ['FDM - runway'];
simulationTypeInv{2,1} = ['SSA - flat'];
simulationTypeInv{1,1} = ['FDM - flat'];

pathLossIndra_alongX(rx_xArr4,rx_zArr4,eFieldAlongTot2Inv,...
    frequency,simulationTypeInv,' ',titleFig5,' ')

ax1 = gca;

% Plotting the surface with logarithmic scale:
fig = figure();
     
semilogx((xVect(startIndex2:length(xVect))),zSurfaceNorm...
     (startIndex2:length(xVect))); 

legend('Runway surface profile','Location','SouthWest');
xlabel('Distance [m]');
ylabel('Height [m]');
plotwidth = 560;
plotheight = 200;
set(fig, 'Position', [500 100 plotwidth plotheight]);
grid on
titleFig6 = ['Luton_results/',...
    'Runway_profile_small_Luton.png'];
saveas(fig,titleFig,'png');
titleFig6 = titleFig6(1:(length(titleFig6)-4));
titleFig6 = horzcat(titleFig6,'.pdf');
%print (fig, '-dpdf', titleFig);
save2pdf(titleFig6);

% Plotting the line for field observation:
fig = figure();
obsLine = zSurfaceNorm +(15/deltaX);
plot(xVect,obsLine,'--k'); 
hold on
plot(xVect,zSurfaceNorm,'k'); 
hold on
X_slope = [0 1500];
Z_slope = [13.525 32.007];
plot(X_slope,Z_slope);
legendName = cell(3,1);
legendName{1,1} = 'Observation points';
legendName{2,1} = 'Runway surface profile';
legendName{3,1} = 'Line-of-sight line';
legend(legendName,'Location','SouthWest');
xlabel('Distance [m]');
ylabel('Height [m]');
plotwidth = 560;
plotheight = 300;
set(fig, 'Position', [500 100 plotwidth plotheight]);
grid on
titleFig6 = ['Luton_results/',...
    'Runway_profile_lineOfSight_Luton.png'];
saveas(fig,titleFig,'png');
titleFig6 = titleFig6(1:(length(titleFig6)-4));
titleFig6 = horzcat(titleFig6,'.pdf');
%print (fig, '-dpdf', titleFig);
save2pdf(titleFig6);