% parabolicEquationBC.m

% Testing for boundary conditions added:

% testSquareMatrix.m: Tests the hypothesis regarding square matrices easier
%                     give stability, for a maximum delta x. The range
%                     being calculated depends on delta z. This is for the
%                     finite-difference method.

testResults = 'AbsorptionLayerTest/FDM_testresults_parabolicEquationBC.txt';
fileID = fopen(testResults, 'w');
fprintf(fileID, '%s\r\n', 'Testing stability for square matrices where delta x = 1.5m');
fprintf(fileID, '%s\r\n', ' ');



theta0 =0;% pi/4;% 0;
beta = pi/15; % 0.5;
A = 2;
frequency = 100*10^6;
fprintf(fileID, '%s', 'Frequency: ');
fprintf(fileID, '%e Hz\r\n', frequency);

g = 1.3;
fprintf(fileID, '%s', 'g value: ');
fprintf(fileID, '%4.3f\r\r\n', g);

% The delta x value:
deltaX =1; % 1.5;
fprintf(fileID, '%s', 'Delta x: ');
fprintf(fileID, '%4.3f\r\r\n', deltaX);
maxX = 3000;
fprintf(fileID, '%s', 'Max x: ');
fprintf(fileID, '%4.3f\r\r\n', maxX);
fprintf(fileID, '%s\r\n', ' ');
fprintf(fileID, '%s\r\n', ' ');

% The delta z value:
deltaZarr = 1.5; %0.09; %[0.5 0.3]; % [0.6 0.55 0.525 0.5 0.475 0.45 0.4 0.35 0.3]; %[0.3 0.2 0.1 0.09];
parfor b = 1: length(deltaZarr)
    deltaZ = deltaZarr(b);
    fprintf(fileID, '%s', 'Delta z: ');
    fprintf(fileID, '%4.3f\r\n', deltaZ);
    
    
    antHeight = [7.5]; %[7.5]; % 10];
    for a =1:length(antHeight)
        zs = antHeight(a);
        fprintf(fileID, '%s\r\n', ' ');
        fprintf(fileID, '%s', '    Antenna height: ');
        fprintf(fileID, '%4.2f m\r\n', zs);
        maxHinterestHeight =50; % 25;%; %[8 14 50];
        for heightIndex = 1: length(maxHinterestHeight) %4:5 % 2:3
            maxHinterest = maxHinterestHeight(heightIndex)*zs; % heightIndex*zs;
            fprintf(fileID, '%s', '        Maximum height of interest (in terms of multiple of antenna height): ');
            fprintf(fileID, '%4.2f m\r\n', maxHinterestHeight(heightIndex));
            
            
            [zVectFDM,HindexFDM] =createZvectAbsorptionLayer(maxHinterest,deltaZ);
            initialFieldFDM = createInitialField(zs,theta0,beta,zVectFDM,A, frequency,'circular');
            numZpoints = length(zVectFDM);
            figure()
            %plot(abs(initialFieldFDM))

            % Creating the x vector:
            xVect = verticalVector([0:deltaX:maxX]);
            
            fprintf(fileID, '%s', '        Distance: ');
            fprintf(fileID, '%5.1f\r\n', xVect(numZpoints));
            
            
            numIterations = ceil(maxX/xVect(numZpoints));
            xVectTot = xVect;
            tic
            [uValuesFDMalne,maxEigVal] = FDMAbsorptionLayerNumEfficient2(initialFieldFDM, ...
                zVectFDM,xVect,HindexFDM,frequency,g);
            uValuesTot = uValuesFDMalne;
            toc
%             for it = 1: numIterations 
%                 initField = uValuesFDMalne(:,numZpoints);
%                 tic
%             [uValuesFDMalne,maxEigVal] = FDMAbsorptionLayerNumEfficient(initField, ...
%                 zVectFDM,xVect,HindexFDM,frequency,g);
%             toc
%             uValuesTot = horzcat(uValuesTot,uValuesFDMalne);
%             xVectTot = vertcat(xVectTot, (xVect + xVectTot(length(xVectTot))));
%             end
            
            
            
            fprintf(fileID, '%s', '        Maximum eigenvalue:');
            fprintf(fileID, '%4.6f\r\n', maxEigVal);
            fprintf(fileID, '%s\r\n', ' ');
            
            deltaZstr = num2str(deltaZ);
            yText = strcat('Height above surface, \Delta z ',deltaZstr);
%             
            
            eField2 = uValuesTot(ceil(HindexFDM*(zs/maxHinterest)):ceil(HindexFDM*(zs/maxHinterest)),:);
            simulationType2 = cell(1);
            simulationType2{1} = 'Finite-Difference Method';
            colors = ['g','r'];
            tx = zs;
            rx = zs;
            title = ['Finite-difference method, antenna height:',' ',num2str(antHeight(a))];
            pathLossFlat(tx,rx,xVectTot,eField2,zs, frequency, simulationType2,title)
            
            figure()
            %contourf(abs(uValuesFiniteDiffMethod).^2,50)
            uValuesAux = uValuesTot;
            uValuesAux(abs(uValuesAux)<10^-3) = 10^-3;
            contourf(xVect,zVectFDM,10.*log10(abs(uValuesAux.^2)),20)
            %contourf(10.*log10(abs(uValuesTot(1:1*HindexFDM,:)).^2),20)
            title('E-field - Finite-Difference Method - square matrix')
            xlabel('The surface');
            ylabel(yText); %'Height above surface, \Delta z = %2.1f', deltaZ)
            grid on

            
        end
    end
end


fclose(fileID);


%%

