% Testing functions and algorithms used in the Parabolic Equation Method


%% Testing functions
triDiagMat = [1 0 0 0 0 0 0 0; 1 2 1 0 0 0 0 0; 0 1 9 1 0 0 0 0; ...
    0 0 1 5 1 0 0 0; 0 0 0 1 4 1 0 0; 0 0 0 0 1 7 1 0; ...
    0 0 0 0 0 1 2 1; 0 0 0 0 0 0 0 1 ]
resultIn = verticalVector([1 4 6 2 7 7 5 3])
tic
unknowns = solveTridiagonalMatrix(triDiagMat,resultIn)
toc
testRes = triDiagMat*unknowns

%tic
%%
[subDiagonalEli,resultOut] = subDiagonalElimination(triDiagMat,resultIn)
%%
%toc
tic
 X = linsolve(triDiagMat,resultIn);
 toc
 testRes2 = triDiagMat*X
%  testRes = subDiagonalEli*resultOut
%%
[supDiagonalEli,resultOut] = supDiagonalElimination(triDiagMat,resultIn)
testRes = supDiagonalEli*resultOut

%% Test createInitial field, finiteDifferendeMethod, and splitStepAlgorithm
%close all
theta0 = 0;
beta = 0.5;
A = 1;
frequency = 100*10^6;
lastZ = 100; % Maximum height of consideration
stepFDM =0.09; % Step values for Finite Difference Method (if convergence problem, set stepFDM=0.05
maxHinterest = 50;
stepSSA =1.3;  % Step value for Step-Split Algorithm
transformSizeL = 2*ceil(maxHinterest/stepSSA);
zs = lastZ./4; % m - Height of antenna
zVectFDM = verticalVector([0:stepFDM:lastZ]);
zVectSSA = verticalVector([0:stepSSA:transformSizeL]);

% Test createInitialField
initialFieldFDM = createInitialField(zs,theta0,beta,zVectFDM,A, frequency);
initialFieldSSA = createInitialField(zs,theta0,beta,zVectSSA,A, frequency);
%plot(zVect,initialField);

% Test finiteDifferendeMethod
lastX = 300; % The length of the runway
stepX = 1;
%xVect = verticalVector(0:stepX:lastX);
xVectSSA = verticalVector(0:stepX:lastX);
xVectFDM = verticalVector(0:stepX:lastX);
%uValuesFiniteDiffMethod = finiteDifferenceMethod(initialFieldFDM,zVectFDM,xVectFDM,frequency);

% Test splitStepAlgorithm
uValuesSplitStep = splitStepAlgorithm(initialFieldSSA,zVectSSA,xVectSSA,frequency);

%uValuesSplitStep = splitStepAlgorithmAbsorptionLayer(initialFieldSSA,zVectSSA,xVectSSA,maxHinterest,frequency);

%% Plotting results from the previous cell
close all
%a=uValuesFiniteDiffMethod(:,lastX)
%b=uValuesSplitStep(:,lastX)
%lastZ
% size_uValues1= size(uValuesFiniteDiffMethod)
% size_uValues2= size(uValuesSplitStep)

figure(1)
semilogy((abs(uValuesSplitStep(:,xVectSSA(length(xVectSSA))).^2)))
xlabel('Receiver height');
ylabel('E-field strength at receiver');

% figure(2)
% semilogy(xVect,(abs(uValuesSplitStep(zs,:)).^2))
% hold on
% above = 1;
% semilogy(xVect,(abs(uValuesSplitStep(zs+above,:)).^2),'g')
% hold on
% below = 1;
% semilogy(xVect,(abs(uValuesSplitStep(zs-below,:)).^2),'r')
% title('Field strength along the surface (flat surface)');
% xlabel('Range along the surface [m]')
% ylabel('Field strength');
% zsStr = sprintf('Transmitter %d m above the surface',zs);
% zsStrAbove = sprintf('Transmitter %d m above the surface',zs+above);
% zsStrBelow = sprintf('Transmitter %d m above the surface',zs-below);
% legend(zsStr,zsStrAbove,zsStrBelow);

figure(3)
contourf(10.*log10(abs(uValuesSplitStep).^2),30)
%contourf(abs(uValuesSplitStep).^2,50)
title('E-field - Split-step Algorithm')
xlabel('The surface');
ylabel('Height above surface')
grid on

figure(4)
%contourf(abs(uValuesFiniteDiffMethod).^2,50)
contourf(10.*log10(abs(uValuesFiniteDiffMethod).^2),20)
title('E-field - Finite-Difference Method')
xlabel('The surface');
ylabel('Height above surface')
grid on

% figure(5)
% %contourf(abs(uValuesFiniteDiffMethod).^2,50)
% contourf(10.*log10(abs(uValuesFiniteDiffMethod).^2),20)
% title('E-field - Finite-Difference Method')
% xlabel('The surface');
% ylabel('Height above surface')
% grid on


figure(6)
plot(xVectSSA,10.*log10(abs(uValuesSplitStep(26,:)).^2),'-b')%(lastZ/2 +1,:)).^2),'+b')
hold on
plot(xVect,10.*log10(abs(uValuesFiniteDiffMethod(501,:)).^2),'r')%(lastZ/2+1,:)).^2),'r')
legend('Split-step','Finite difference method')

grid on

%% Testing Split-step algorithm and Finite-difference method
% with absorption layer

maxHinterest = 60;
zs = 7.5;
theta0 = 0;
beta = 0.5;
A = 1;
frequency = 100*10^6;

% Setting deltaZ:
deltaZfdm = 0.05;
deltaZssa = 0.9;

% Creating zVect and initial field:
[zVectFDM,HindexFDM] =createZvectAbsorptionLayer(maxHinterest,deltaZfdm);
zVectFDM = zVectFDM(1:3000,1);
initialFieldFDM = createInitialField(zs,theta0,beta,zVectFDM,A, frequency);
%plot(abs(initialFieldFDM))

[zVectSSA,HindexSSA] =createZvectAbsorptionLayer(maxHinterest,deltaZssa);
initialFieldSSA = createInitialField(zs,theta0,beta,zVectSSA,A, frequency);

% Creating xVect:
deltaX = 1;
maxX = 3000; %10000;
g = 1.3;
xVect = verticalVector([0:deltaX:maxX]);

% Calculating the field:

tic
[uValuesFDMalFlatNumeff,maxEigVal] = FDMAbsorptionLayerNumEfficient(initialFieldFDM, ...
                zVectFDM,xVect,HindexFDM,frequency,g);
toc
tic
[uValuesFDMalFlat] = finiteDifferenceMethodAbsorptionLayer(initialFieldFDM, ...
     zVectFDM,xVect,maxHinterest,frequency);
toc
maxEigVal
uValuesSSAalFlat = splitStepAlgorithmAbsorptionLayer(initialFieldSSA,zVectSSA,xVect, ...
    HindexSSA,frequency);
%%
close all;
eField = uValuesSSAalFlat(ceil(HindexSSA*0.125),:);
simulationType = 'Finite-Difference Method';
pathLossFlat(tx,rx,xVect,eField,zs, frequency, simulationType)


%% Plotting results from the previous section
close all

figure(1)
contourf(10.*log10(abs(uValuesSSAalFlat).^2),30)
%contourf(abs(uValuesSplitStep).^2,50)
title('E-field - Split-step Algorithm')
xlabel('The surface');
ylabel('Height above surface')
grid on
%%
figure(2)
%contourf(abs(uValuesFiniteDiffMethod).^2,50)
contourf(10.*log10(abs(uValuesFDMalFlat).^2),20)
title('E-field - Finite-Difference Method')
xlabel('The surface');
ylabel('Height above surface')
grid on

%%
figure(3)
%contourf(abs(uValuesFiniteDiffMethod).^2,50)
contourf(10.*log10(abs(uValuesFDMalne).^2),20)
title('E-field - Finite-Difference Method')
xlabel('The surface');
ylabel('Height above surface')
grid on


%% Irregular terrain
close all
zs = 3;
theta0 = 0;
beta = 2;
A = 1;
frequency = 100*10^6;

maxHeight = 60;
zStepSSA = 0.5;
zVectSSA = verticalVector([0:zStepSSA:maxHeight]);

zStepFDM = 0.1;
zVectFDM = verticalVector([0:zStepFDM:maxHeight]);


% The irregular terrain:
Xn = [0, 150, 3000];
Zn = [0, 10, 0];
[xVect,zSurfaceVect] = interpolate(Xn,Zn,frequency,'linear',1);

initialFieldSSA = createInitialField(zs,theta0,beta,zVectSSA,A, frequency,'gaussian1');
%plot(initialFieldSSA)
[zVectSSAal,HindexSSA] =createZvectAbsorptionLayer(maxHeight,zStepSSA);
initialFieldSSAal = createInitialField(zs,theta0,beta,zVectSSAal,A, frequency,'gaussian1');

initialFieldFDM = createInitialField(zs,theta0,beta,zVectFDM,A, frequency,'gaussian1');

 [zVectFDMal,HindexFDM]=createZvectAbsorptionLayer(maxHeight,zStepFDM);
 initialFieldFDMal = createInitialField(zs,theta0,beta,zVectFDMal,A, frequency,'gaussian1');


 uValuesSSAIrrTerrain=splitStepAlgorithmIrregularTerrain ...
     (initialFieldSSA,zVectSSA,xVect,zSurfaceVect,frequency);
 
 uValuesSSAitAbsorptionLayer = SSAirregularTerrainAbsoptionLayer(...
     initialFieldSSAal,zVectSSAal,xVect,zSurfaceVect,HindexSSA,frequency);
 
%  uValuesFDMIrrTerrain = finiteDifferenceMethodIrregularTerrain(initialFieldFDM, ...
%     zVectFDM,xVect,zSurfaceVect,frequency);
%%
tic
uValuesFDMitAbsorptionLayer = FDMirregularTerrainAbsorptionLayer(initialFieldFDMal, ...
    zVectFDMal,xVect,zSurfaceVect,HindexFDM,frequency);
toc
save('IrregularTerrainCellResults2')

%% Plotting results from the previous section
%close all

figure(1)
contourf(10.*log10(abs(uValuesSSAIrrTerrain).^2),30)
%contourf(abs(uValuesSplitStep).^2,50)
title('E-field - Split-step Algorithm - Irregular terrain')
xlabel('The surface');
ylabel('Height above surface')
grid on

figure(2)
contourf(10.*log10(abs(uValuesSSAitAbsorptionLayer).^2),30)
%contourf(abs(uValuesSplitStep).^2,50)
title('E-field - Split-step Algorithm - Irregular terrain, absorption layer')
xlabel('The surface');
ylabel('Height above surface')
grid on


figure(3)
contourf(10.*log10(abs(uValuesFDMIrrTerrain).^2),10)
%contourf(abs(uValuesSplitStep).^2,50)
title('E-field - Finite-difference method - Irregular terrain')
xlabel('The surface');
ylabel('Height above surface')
grid on

figure(4)
contourf(10.*log10(abs(uValuesFDMitAbsorptionLayer(1:1000,:)).^2),10)
%contourf(abs(uValuesSplitStep).^2,50)
title('E-field - Finite-difference method - Irregular terrain, absorption layer')
xlabel('The surface');
ylabel('Height above surface')
grid on

%% Testing the more numerical efficient alogrithms

maxHinterest = 60;
zs = 20;
theta0 = 0;
beta = 0.5;
A = 1;
frequency = 100*10^6;

% Setting deltaZ:
deltaZfdm = 0.2;

% Creating zVect and initial field:
[zVectFDM,HindexFDM] =createZvectAbsorptionLayer(maxHinterest,deltaZfdm);
%zVectFDM = zVectFDM(1:3000,1);
initialFieldFDM = createInitialField(zs,theta0,beta,zVectFDM,A, frequency);
%plot(abs(initialFieldFDM))

% Creating xVect:
deltaX = 1.5;
maxX = 2000;
xVect = verticalVector([0:deltaX:maxX]);
disp('ok');
%%
uValuesFDMalne = FDMAbsorptionLayerNumEfficient(initialFieldFDM, ...
    zVectFDM,xVect,HindexFDM,frequency);
%%
close all
matrix2 = 10.*log10(abs(uValuesFDMalne).^2);
%%
figure(1)
%contourf(matrix2(1:1000,1:3000),30);
contourf(10.*log10(abs(uValuesFDMalne).^2),30)
%contourf(abs(uValuesSplitStep).^2,50)
title('E-field - FDM - Numerical Efficient');
xlabel('The surface');
ylabel('Height above surface');
grid on

%%
%uValMat = [];
i = 0;
for g = 0.00001:0.00001 % 0.001:0.003:0.01
    i = i+1;
uValues = FDMAbsorptionLayerNumEfficient(initialFieldFDM, ...
    zVectFDM,xVect,HindexFDM,frequency,g);
%uValMat(i) = uValues;
figure()
contourf(10.*log10(abs(uValues).^2),30)
title('E-field - FDM - Numerical Efficient');
xlabel('The surface');
ylabel('Height above surface');
end









%% Test discrete sine transform (forth and back)
x1Sample = [1; 2; 3; 4; 5; 6; 7; 8; 9; 10];
x2Sample =  [0.1576 + 0.6557i; 0.9706 + 0.0357i; 0.9572 + 0.8491i; ...
   0.4854 + 0.9340i; 0.8003 + 0.6787i; 0.1419 + 0.7577i; ...
   0.4218 + 0.7431i; 0.9157 + 0.3922i; 0.7922 + 0.6555i; ...
   0.9595 + 0.1712i];

% Discrete sine transform:
dst1 = discreteSineTrans(x1Sample);
dst2 = discreteSineTrans(x2Sample);


% Inverse discrete sine transform
idst1 = inverseDiscreteSineTrans(dst1);
idst2 = inverseDiscreteSineTrans(dst2);

close all
figure(1)
plot(x1Sample,'+b')
hold on
plot(idst1,'r')
legend('Original', 'IDST');
title('Discrete Sine Tranform');

figure(2)
plot(abs(x2Sample),'+b')
hold on 
plot(abs(idst2),'r')
legend('Original', 'IDST');
title('Discrete Sine Transform'); 

% Test vs the results from the matlab inbuilt functions:

% Discrete Sine Transform
dst1Matlab =  [38.2533; -18.7313; 12.0433; -8.5582; 6.3473; -4.7658; ...
    3.5346; -2.5118; 1.6149; -0.7908];
dst2Matlab =  [4.4410 + 4.4921i; 0.0034 + 0.2724i; 2.4890 + 0.2011i; ...
    -1.0252 - 0.3027i; 0.1203 + 0.0629i; -0.3498 + 0.2821i; ...
    -0.4552 + 0.7635i; -1.3412 + 1.3523i; 0.4341 + 0.1842i; ...
    0.3028 + 0.5650i];

figure(3)
plot(dst1,'+b')
hold on
plot(dst1Matlab,'r');
legend('Self-written dst','matlab dst');
title('Discrete Sine Transform, sample 1');

figure(4)
plot(abs(dst2),'+b')
hold on
plot(abs(dst2Matlab),'r');
legend('Self-written dst','matlab dst');
title('Discrete Sine Transform, sample 2');


%% Test analogy to Fourier Transform


x1Sample = [1; 2; 3; 4; 5; 6; 7; 8; 9; 10]
x2Sample =  [0.1576 + 0.6557i; 0.9706 + 0.0357i; 0.9572 + 0.8491i; ...
   0.4854 + 0.9340i; 0.8003 + 0.6787i; 0.1419 + 0.7577i; ...
   0.4218 + 0.7431i; 0.9157 + 0.3922i; 0.7922 + 0.6555i; ...
   0.9595 + 0.1712i];

% Discrete Fourier Transform
fft1 = fft(x1Sample);
fft2 = fft(x2Sample);

% Inverse Discrete Fourier Transform
ifft1 = ifft(fft1);
ifft2 = ifft(fft2);

close all
figure(1)
plot(x1Sample,'+b')
hold on
plot(ifft1,'r')
legend('Original', 'IFFT');
title('Discrete Fourier Transform');

figure(2)
plot(abs(x2Sample),'+b')
hold on 
plot(abs(ifft2),'r')
legend('Original', 'IFFT');
title('Discrete Fourier Transform');

%% Test Symbolic Evaluation
syms b m
mat = sym([1 0 0 0 0 0 0 0 0 0; ...
    -1 b -1 0 0 0 0 0 0 0; ...
    0 -1 b -1 0 0 0 0 0 0; ...
    0 0 -1 b -1 0 0 0 0 0; ...
    0 0 0 -1 b -1 0 0 0 0; ...
    0 0 0 0 -1 b -1 0 0 0; ...
    0 0 0 0 0 -1 b -1 0 0; ...
    0 0 0 0 0 0 -1 b -1 0; ...
    0 0 0 0 0 0 0 -1 b -1; ...
    0 0 0 0 0 0 0 0 0 1]) 
result = expand(mat^10)
a = 0.05;

mat1 = [1 0 0 0 0 0 0 0 0 0; ...
    -1 a -1 0 0 0 0 0 0 0; ...
    0 -1 a -1 0 0 0 0 0 0; ...
    0 0 -1 a -1 0 0 0 0 0; ...
    0 0 0 -1 a -1 0 0 0 0; ...
    0 0 0 0 -1 a -1 0 0 0; ...
    0 0 0 0 0 -1 a -1 0 0; ...
    0 0 0 0 0 0 -1 a -1 0; ...
    0 0 0 0 0 0 0 -1 a -1; ...
    0 0 0 0 0 0 0 0 0 1];
result1 = mat1^1000




























