% parabolicEquation_SSA_FDM.m: Script running simulations with varying
%                              delta x and delta x values over a flat 
%                              surface using the SSA and FDM.   

clear all

% Setting the parameters:
theta0 = 0; 
beta =pi/18; 
A = 1;
frequency = 110*10^6;
maxX = 3000;
numPtsAbsoptionLayer = 150;

deltaZarr =[0.5 1 1.3];
antHeight = [15];
deltaXvect =[0.5 1 1.3]; 
for a =1:length(antHeight)
    
    
    zs = antHeight(a);
    maxHinterestHeight = 10;
    numElts = 2;
    simulationType = cell(numElts,1);
    
    % Looping over the delta x values:
    for n = 1:length(deltaXvect)
        clear eFieldTot
        counter = 0;
        doubleCounter = 0;
        
        maxHinterestHeight = 10;
        
        for heightIndex = 1: length(maxHinterestHeight)
            
            % Looping over the delta z values
            for  b = 1: length(deltaZarr)
                deltaZ = deltaZarr(b);
                
                counter = counter +1;
                doubleCounter = doubleCounter +1;
                deltaX = deltaXvect(n);
                xVect = verticalVector([0:deltaX:maxX]);
                maxHinterest = 350 +zs;
                
                % Creating initial field:
                [zVectFDM,HindexFDM] =createZvectAbsorptionLayer2(...
                    maxHinterest,deltaZ,numPtsAbsoptionLayer);
                initialFieldFDM = createInitialField(zs,theta0,beta,...
                    zVectFDM,A, frequency,'gaussian1');
                numZpoints = length(zVectFDM);
                
                % Calculating field:
                tic
                uValuesSplitStep =splitStepAlgorithmAbsorptionLayer(...
                    initialFieldFDM,zVectFDM,xVect, ...
                    HindexFDM,frequency,numPtsAbsoptionLayer);
                toc
                tic
                [uValuesFDMalne,maxEigVal]=...
                    FDMAbsorptionLayerNumEfficient2(initialFieldFDM, ...
                    zVectFDM,xVect,HindexFDM,frequency,...
                    numPtsAbsoptionLayer);
                toc
                
                deltaZstr = num2str(deltaZ*10);
                yText = strcat('Height above surface [m]');
                
                % Extract the simulated fields at the height of interest:
                eFieldSSA=uValuesSplitStep(...
                    ceil(HindexFDM*(zs/maxHinterest)),:);
                eFieldFDM=uValuesFDMalne(...
                    ceil(HindexFDM*(zs/maxHinterest)),:);
               
                simulationType{doubleCounter,1} = ['SSA: \Deltax=',...
                    num2str(deltaX),'m, \Deltaz=',num2str(deltaZ),'m'];
                doubleCounter = doubleCounter +1;
                simulationType{doubleCounter,1} = ['FDM:\Deltax=',...
                    num2str(deltaX),'m, \Deltaz=',num2str(deltaZ),'m'];
                tx = zs;
                rx = zs;
                
                if counter == 1
                    eFieldTot = eFieldSSA;
                    eFieldTot = vertcat(eFieldTot,eFieldFDM);
                else
                    eFieldTot = vertcat(eFieldTot,eFieldSSA);
                    eFieldTot = vertcat(eFieldTot,eFieldFDM);
                end
                
                % Plot the simulated fields:
                fig2 = figure('visible','off');               
                uValuesAux = uValuesSplitStep;
                uValuesAux(abs(uValuesAux)<10^-4) = 10^-4;
                contourf(xVect,zVectFDM,10.*log10(abs(uValuesAux.^2)),50)
                hold on
                contour(xVect,zVectFDM,10.*log10(abs(uValuesAux.^2)),50)
               
                xlabel('Distance [m]');
                ylabel(yText);
                grid on
                titleFig = ['Results_to_thesis/SSA_', ...
                    'Deltaz','_',deltaZstr,'_','Deltax_','_', ...
                    num2str(deltaX*10),'_distance_source_abslayer',...
                    num2str(maxHinterest-zs),'freq_',...
                    num2str(frequency/(10^6)),'_deltaDiff.png'];
                saveas(fig2,titleFig,'png');
                
                
                fig = figure('visible','off');
                uValuesAux = uValuesFDMalne;
                uValuesAux(abs(uValuesAux)<10^-4) = 10^-4;
                contourf(xVect,zVectFDM,10.*log10(abs(uValuesAux.^2)),50)
                hold on
                contour(xVect,zVectFDM,10.*log10(abs(uValuesAux.^2)),50)            
                xlabel('Distance [m]');
                ylabel(yText);
                grid on
                titleFig = ['Results_to_thesis/FDM_', ...
                    'Deltaz','_',deltaZstr,'_','Deltax_','_', ...
                    num2str(deltaX*10),'freq_',...
                    num2str(frequency/(10^6)),'_deltaDiff.png'];
                saveas(fig,titleFig,'png');
                
                
                
                
            end
            
            tx =zs;
            rx = zs;
            part1Title = ['SSA and FDM - flat surface'];
            titleFig = ['Results_to_thesis/SSA_FDM_', ...
                'Deltaz','_',deltaZstr,'_','Deltax_','_', ...
                num2str(deltaX*10),'freq_',...
                    num2str(frequency/(10^6)),'_deltaDiff.png'];
            % Comparison between simulated and analytical results:
            pathLossFlat_beamParam(A,tx,rx,xVect,eFieldTot,zs,...
                beta,frequency,simulationType, ' ',titleFig);
            
        end
    end
end





