% ParabolicEquation_noGround.m: Script that simulate free-space only, for
%                               testing the absorption layer.
clear all

% Setting the parameters:
theta0 =0;
beta = pi/15; 
A = 2;
frequency = 100*10^6;
deltaX =1; 
maxX = 5000;
maxHinterestHeight =20; 
numLayersArray  =10;
numPointsPerLayerArray =20; 
numElts = length(numPointsPerLayerArray);
numPointsInLayer =100;
deltaZarr = 1; 

% Looping over the deltaZ values in question:
for b = 1: length(deltaZarr)
    deltaZ = deltaZarr(b);
        
    antHeight =10;
    
    % Looping over the antenna heights in question:
    for a =1:length(antHeight)
        counter = 0;
        simulationType = cell(numElts,1);
        for heightIndex = 1: length(maxHinterestHeight) 
            for n = 1: length(numLayersArray)
                
                for m = 1: length(numPointsPerLayerArray)
                    counter = counter +1;
                    numLayers = numLayersArray(n);
                    numPointsPerLayer = numPointsPerLayerArray(m);
                    
                    zs = antHeight(a);
                      
                    maxHinterest =260;                 
                    
                    % Creating z-vector with absorption layer:
                    [zVectFDM,HindexFDM] =createZvectAbsorptionLayer2( ...
                        maxHinterest, deltaZ,numPointsInLayer);
                    
                    % Creatin initial field:
                    initialFieldFDM =createInitialField(zs,theta0,beta,...
                        zVectFDM, A, frequency,'gaussian1');
                    numZpoints = length(zVectFDM);
                    
                    
                    % Creating the x vector:
                    xVect = verticalVector([0:deltaX:maxX]);
                    
                                      
                    numIterations = ceil(maxX/xVect(numZpoints));
                    xVectTot = xVect;
                    L = length(zVectFDM);
                    sourceIndex = ceil(((L/deltaZ) +1)*(zs/L));
                    
                    % Simulating the field:
                    tic
                    [uValuesFDMalne,maxEigVal,antennaSourceIndex] = ...
                        FDMnoGround(initialFieldFDM, ...
                        zVectFDM,xVect,HindexFDM,frequency,...
                        numPointsInLayer,sourceIndex);
                    uValuesTot = uValuesFDMalne;
                    
                    toc
            
                    deltaZstr = num2str(deltaZ);
                    yText = 'Height distance [m]';
                    %
                    
                    eField = uValuesTot(antennaSourceIndex,:);
                    simulationType{counter,1} = ['FDM'];
                    
                    if counter == 1
                        eFieldTot = eField;
                    else
                        
                        eFieldTot = vertcat(eFieldTot,eField);
                    end
                    
                    tx = zs;
                    rx = zs;
                    
                    % Plotting the simulated field:
                    fig = figure('visible','off');
                    uValuesAux = uValuesTot;
                    uValuesAux(abs(uValuesAux)<10^-4) = 10^-4;
                    contourf(10.*log10(abs(uValuesAux.^2)),50)
                    hold on
                    contour(10.*log10(abs(uValuesAux.^2)),50)
                    
                    part1Title = ['FDM - absorption layer test'];
                    part12Titile = [ ' ','\Deltaz = ',' ',deltaZstr,...
                        'm, ','\Deltax = ',' ', ...
                        num2str(deltaX),'m,'];
                    part21Title = ['Distance from center point of ',...
                        'source to beginning of absorption layer: ',...
                        num2str(maxHinterest-zs),'m' ];
                    part2Title=['Number of points in absorption layer:',...
                        ' ', num2str(numPointsInLayer)];
                 
                    titleVal2 = {part1Title;part12Titile;part21Title;...
                        part2Title};
                    title(titleVal2)
                    xlabel('Distance [m]');
                    ylabel(yText); 
                    grid on
                    titleFig = [...
                        'FDM_noGround/FDM_AbsorptioLayerTest_ant_h_',...
                        '_',num2str(antHeight(a)), ...
                        '_','Deltaz','_',deltaZstr,'_','Deltax_','_', ...
                        num2str(deltaX),'_max_h_',...
                        num2str(maxHinterestHeight(heightIndex)), ...
                        'num_points_layer', num2str(numPointsInLayer),...
                        '.png'];
                    saveas(fig,titleFig,'png');
                end
            end
        end
        part1Title = ['FDM - absorption layer test'];
        part12Titile = [ ' ','\Deltaz = ',' ',deltaZstr,...
            'm, ','\Deltax = ',' ', ...
            num2str(deltaX),'m,'];
        part21Title = ['Distance from center point of ',...
            'source to beginning of absorption layer: ',...
            num2str(maxHinterest-zs),'m'];
        part2Title=['Number of points in absorption layer:',...
            ' ', num2str(numPointsInLayer)];
        
        plotTitle = {part1Title;part12Titile;part21Title;...
            part2Title};
        
        titleFig = ['FDM_noGround/FDM_AbsorptioLayerTest_ant_h_','_',...
            num2str(antHeight(a)), ...
            '_','Deltaz','_',deltaZstr,'_','Deltax_','_', ...
            num2str(deltaX),'_max_h_',...
            num2str(maxHinterestHeight(heightIndex)), ...
            'num_points_layer', num2str(numPointsInLayer),...
            'newAbsLAyer4','.png'];
        tx = 10000;
        rx = 10000;
        
        % Comparing the simulated free-space loss with analytical
        % free-space loss:
        freeSpaceLoss_beamParam(A,tx,rx,xVect,eFieldTot,zs,...
            beta,frequency,simulationType, plotTitle,titleFig);
        
        
    end
end



