% SSA_FDM_indra_r_loss.m: Script calcualting the electric field along a
% flat surface and compare the results with analytical results from Indra.
% In the computations, additional loss, (1/r) is added in order to ''make''
% a 3D model.

clear all

% Setting the parameters:
theta0 = 0;
beta =(55/(2*360))*(2*pi); 
A =10; 
frequency = 110*10^6;

deltaX = 1;
maxX = 3000;
compareDistance = 1000; 
numPtsAbsoptionLayer = 150;
deltaZ = 1;
antHeight = 3;
deltaXvect = [1];
counter = 0;
doubleCounter = 0;
maxHinterestHeight = 10;
numElts = 2;
simulationType = cell(numElts,1);

counter = counter +1;
doubleCounter = doubleCounter +1;

xVect = verticalVector([0:deltaX:maxX]);
maxHinterest = 350 +antHeight;

% Creating initial field:
[zVectFDM,HindexFDM] =createZvectAbsorptionLayer2(...
    maxHinterest,deltaZ,numPtsAbsoptionLayer);
initialFieldFDM = createInitialField(antHeight,theta0,beta,...
    zVectFDM,A, frequency,'gaussian1');

% Finding the gain of the used beam [dBi]:
maxValueInitField = max(initialFieldFDM);
sumInitField = sum(initialFieldFDM);
findRes = find(abs(initialFieldFDM)>0);
numResPts = length(findRes);
isotropicSource = sumInitField/numResPts;
dbiGain = maxValueInitField/isotropicSource

numZpoints = length(zVectFDM);

% Calculating field with 1/r-loss added to the results:
tic
uValuesSplitStep =SSA_addRloss(...
    initialFieldFDM,zVectFDM,xVect, ...
    HindexFDM,frequency,numPtsAbsoptionLayer,antHeight);
toc
tic
[uValuesFDMalne]=...
    FDM_addRloss(initialFieldFDM, ...
    zVectFDM,xVect,HindexFDM,frequency,numPtsAbsoptionLayer,antHeight);
toc

deltaZstr = num2str(deltaZ);
yText = strcat('Height above surface [m]');

% Extracting the simulated values at the height of the anntenna:
eFieldSSA=uValuesSplitStep(ceil(HindexFDM*(antHeight/maxHinterest)),:);
eFieldFDM=uValuesFDMalne(ceil(HindexFDM*(antHeight/maxHinterest)),:);

simulationType{1,1} = ['SSA'];
doubleCounter = doubleCounter +1;
simulationType{2,1} = ['FDM'];
tx = antHeight;
rx = antHeight;

eFieldTot = eFieldSSA;
eFieldTot = vertcat(eFieldTot,eFieldFDM);


%Plot SSA figuure:
fig2 = figure('visible','off');
part1Title = ['Split-Step Algorithm - flat surface'];
titleVal2 = {part1Title};

uValuesAux = uValuesSplitStep;
uValuesAux(abs(uValuesAux)<10^-11) = 10^-11;
contourf(xVect,zVectFDM,10.*log10(abs(uValuesAux.^2)),50)
hold on
contour(xVect,zVectFDM,10.*log10(abs(uValuesAux.^2)),50)
%title(titleVal2)
xlabel('Distance [m]');
ylabel(yText);
grid on
titleFig = ['Indra_r_loss_added/SSA_flat_rLoss.png'];
saveas(fig2,titleFig,'png');

%Plot FDM figure:
fig = figure('visible','off');
part1Title = ['Finite-Difference Method - flat surface'];
titleVal2 = {part1Title};

uValuesAux = uValuesFDMalne;
uValuesAux(abs(uValuesAux)<10^-11) = 10^-11;
contourf(xVect,zVectFDM,10.*log10(abs(uValuesAux.^2)),50)
hold on
contour(xVect,zVectFDM,10.*log10(abs(uValuesAux.^2)),50)
%title(titleVal2)
xlabel('Distance [m]');
ylabel(yText);
grid on
titleFig = ['Indra_r_loss_added/FDM_flat_rLoss.png'];
saveas(fig,titleFig,'png');
% 
% Comparing with results from Indra (along the surface):
tx =antHeight;
rx = antHeight;
rx_xArr = xVect;
rx_zArr = ones(length(xVect),1).*antHeight;
part1Title = ['SSA and FDM - flat surface - along the surface'];
plotTitle = ' '; 
titleFig = ['Indra_r_loss_added/SSA_FDM_flat_along_surface.png'];
filename = 'IndraWedge2_results/LPDA-u-kile-2_E.xls';

pathLossIndra_alongX(rx_xArr,rx_zArr,eFieldTot,frequency,...
    simulationType, plotTitle,titleFig,filename);

% Comparing with results from Indra with varying height at the end of the
% surface:
xIndex = find(xVect >= 1000,1);
eFieldSSA1 = zeros(1,length(zVectFDM));
eFieldFDM1 = zeros(1,length(zVectFDM));
for i = 1:length(zVectFDM)
    eFieldSSA1(1,i) = uValuesSplitStep(i,xIndex); 
    eFieldFDM1(1,i) = uValuesFDMalne(i,xIndex); 
end

plotTitle1 = ' '; 

eFieldTot1 = eFieldFDM1;
eFieldTot1 = eFieldSSA1;
eFieldTot1 = vertcat(eFieldTot1,eFieldFDM1);
rx_xArr1 = ones(length(zVectFDM),1).*length(xVect);
rx_zArr1 = zVectFDM;

titleFig1 = ['Indra_r_loss_added/SSA_FDM_flat_height_varying_rLoss.png'];
filename1 = 'IndraWedge_results/LPDA-u-kile_E.xls';

compareHeight = 240;
pathLossFlat_Indra(rx_xArr1,rx_zArr1,eFieldTot1,...
    frequency,simulationType, plotTitle1,titleFig1,filename1,compareHeight)

% Calculating the field strength without any additional 1/r-loss:
tic
uValuesSSA2 =splitStepAlgorithmAbsorptionLayer(...
    initialFieldFDM,zVectFDM,xVect, ...
    HindexFDM,frequency,numPtsAbsoptionLayer);
toc

tic
[uValuesFDM2,maxEigVal]=FDMAbsorptionLayerNumEfficient2(initialFieldFDM,...
    zVectFDM,xVect,HindexFDM,frequency,numPtsAbsoptionLayer);
toc

% Extracting the field strength at 1000 m in the vertical direction;
xIndex = find(xVect >= 1000,1);
eFieldSSA2 = zeros(1,length(zVectFDM));
eFieldFDM2 = zeros(1,length(zVectFDM));
for i = 1:length(zVectFDM)
    eFieldSSA2(1,i) = uValuesSSA2(i,xIndex); %...
    %length(xVect));
    eFieldFDM2(1,i) = uValuesFDM2(i,xIndex); %...
    %length(xVect));
end
eFieldTot2 = eFieldSSA2;
eFieldTot2 = vertcat(eFieldTot2,eFieldFDM2);

part1Title2 = ['SSA and FDM - flat surface - receiver height varying '...
    '- no (1/r)-loss added'];
plotTitle2 =' ';% {part1Title1};

rx_xArr2 = ones(length(zVectFDM),1).*length(xVect);
rx_zArr2 = zVectFDM;

titleFig2 =['Indra_r_loss_added/SSA_FDM_flat_height_varying_no_rLoss.png'];
filename2 = 'IndraWedge_results/LPDA-u-kile_E.xls';
% Tests the plotting the results in the same plot:
compareHeight = 240;
eFieldTot3  = eFieldSSA1;
eFieldTot3 = vertcat(eFieldTot3,eFieldFDM1);
eFieldTot3 = vertcat(eFieldTot3,eFieldSSA2);
eFieldTot3 = vertcat(eFieldTot3,eFieldFDM2);

simulationType2 = cell(4,1);
simulationType2{1,1} = ['SSA - 1/r-loss added'];
simulationType2{2,1} = ['FDM - 1/r-loss added'];
simulationType2{3,1} = ['SSA - no 1/r-loss added'];
simulationType2{4,1} = ['FDM - no 1/r-loss added'];

% Vertical comparison of relative field strengths with different max 
% heights: 
 pathLossFlat_Indra(rx_xArr2,rx_zArr2,eFieldTot3,...
     frequency,simulationType2, plotTitle2,titleFig2,filename2,...
     compareHeight)


 compareHeight = 50;
 titleFig3 =['Indra_r_loss_added/',...
     'SSA_FDM_flat_height_varying_no_rLoss_zoomed.png']; 
  pathLossFlat_Indra(rx_xArr2,rx_zArr2,eFieldTot2,...
     frequency,simulationType, plotTitle2,titleFig3,filename2,...
     compareHeight)


 titleFig3 =['Indra_r_loss_added/',...
     'SSA_FDM_flat_height_varying_no_rLoss_zoomed2.png']; 
  pathLossFlat_Indra_minComp(rx_xArr2,rx_zArr2,eFieldTot2,...
     frequency,simulationType, plotTitle2,titleFig3,filename2,...
 compareHeight)

% Due to an error when first implementing the script, the fields without
% additional loss is calculated ones more:

% Setting the parameters:
theta0 = 0; 
A = 10^(0.9);
frequency = 110*10^6;
deltaX = 1; 
numPtsAbsoptionLayer = 150;

deltaZarr =[1];
antHeight = 3;
deltaXvect = [1]; 

Xn = [0 maxX];
Zn = [0 0];


for a =1:length(antHeight)
    counter = 0;
    doubleCounter = 0;
    
    zs = antHeight(a);
    maxHinterestHeight = 10; 
    numElts = 2;
    simulationType = cell(numElts,1);
    
    % Looping over the delta x values:
    for n = 1:length(deltaXvect)
        
        maxHinterestHeight = 10;
        
        for heightIndex = 1: length(maxHinterestHeight) 
            
            % Looping over the delta z values
            for  b = 1: length(deltaZarr)
                deltaZ = deltaZarr(b);
                
                counter = counter +1;
                doubleCounter = doubleCounter +1;
                deltaX = deltaXvect(n);
                
                maxHinterest = 350 +zs;
                
                % Interpolate the flat surface (algorithm for irregular
                % terrain used):
                [xVect,zSurfaceVect] = interpolate(Xn,Zn,frequency,...
                    'linear',deltaX);
                
                % Create initial field:
                [zVectFDM,HindexFDM] =createZvectAbsorptionLayer2(...
                    maxHinterest,deltaZ,numPtsAbsoptionLayer);
                initialFieldFDM = createInitialField(zs,theta0,beta,...
                    zVectFDM,A, frequency,'gaussian1');
                numZpoints = length(zVectFDM);
                
                % Calculating the fields:
                tic
                uValuesSplitStep3 = SSAirregularTerrainAbsoptionLayer(...
                    initialFieldFDM,zVectFDM,...
                    xVect,zSurfaceVect,HindexFDM,frequency,...
                    numPtsAbsoptionLayer,zs);
                toc
                tic
                uValuesFDM3 = FDMirregularTerrainAbsorptionLayer ...
                    (initialFieldFDM,zVectFDM,xVect,zSurfaceVect,...
                    HindexFDM,frequency,numPtsAbsoptionLayer);
                toc
              
                % Extracting the fields ath the given distance:
                xIndex = find(xVect >= compareDistance,1);
                eFieldSSA3 = zeros(1,length(zVectFDM));
                eFieldFDM3 = zeros(1,length(zVectFDM));
                for i = 1:length(zVectFDM)
                    eFieldSSA3(1,i) = uValuesSplitStep3(i,...
                        xIndex);
                    eFieldFDM3(1,i) = uValuesFDM3(i,...
                        xIndex);
                end
                
                deltaZstr = num2str(deltaZ);
                yText = strcat('Height above surface [m]');
                
                simulationType{doubleCounter,1} = ['SSA'];
                doubleCounter = doubleCounter +1;
                simulationType{doubleCounter,1} = ['FDM'];
                tx = zs;
                rx = zs;
                
                if counter == 1
                    eFieldTot3 = eFieldSSA3;
                    eFieldTot3 = vertcat(eFieldTot3,eFieldFDM3);
                else
                    eFieldTot3 = vertcat(eFieldTot3,eFieldSSA3);
                    eFieldTot3 = vertcat(eFieldTot3,eFieldFDM3);
                end
                
                % Plotting the fields:
                fig2 = figure('visible','on');
                
                part1Title = ['Split-Step Algorithm - wedge'];
                part12Titile = [ ' ','\Deltaz = ',' ',deltaZstr,...
                    'm, ','\Deltax = ',' ', ...
                    num2str(deltaX),'m, Source height: ', ...
                    num2str(antHeight(a)),'m'];
                part21Title = ['Distance from center point of ',...
                    'source to beginning of absorption layer: ',...
                    num2str(maxHinterest-zs),'m' ];
                part2Title=['Number of points in absorption layer:',...
                    ' ', num2str(numPtsAbsoptionLayer)];
                
                titleVal2 = {part1Title;part12Titile;part21Title;...
                    part2Title};
                
                uValuesAux = uValuesSplitStep3;
                uValuesAux(abs(uValuesAux)<10^-4) = 10^-4;
                contourf(xVect,zVectFDM,10.*log10(abs(uValuesAux.^2)),50)
                hold on
                contour(xVect,zVectFDM,10.*log10(abs(uValuesAux.^2)),50)
                %title(titleVal2)
                xlabel('Distance [m]');
                ylabel(yText);
                grid on
                titleFig =['Indra_r_loss_added/SSA_flat_noRloss','.png'];
                saveas(fig2,titleFig,'png');               
                
                fig = figure('visible','on');
                
                part1Title = ['Finite-Difference Method - wedge'];
                part12Titile = [ ' ','\Deltaz = ',' ',deltaZstr,...
                    'm, ','\Deltax = ',' ', ...
                    num2str(deltaX),'m, Source height: ', ...
                    num2str(antHeight(a)),'m'];
                part21Title = ['Distance from center point of ',...
                    'source to beginning of absorption layer: ',...
                    num2str(maxHinterest-zs),'m' ];
                part2Title=['Number of points in absorption layer:',...
                    ' ', num2str(numPtsAbsoptionLayer)];
                
                titleVal2 = {part1Title;part12Titile;part21Title;...
                    part2Title};
                
                uValuesAux = uValuesFDM3;
                uValuesAux(abs(uValuesAux)<10^-4) = 10^-4;
                contourf(xVect,zVectFDM,10.*log10(abs(uValuesAux.^2)),50)
                hold on
                contour(xVect,zVectFDM,10.*log10(abs(uValuesAux.^2)),50)
                %title(titleVal2)
                xlabel('Distance [m]');
                ylabel(yText);
                grid on
                titleFig =['Indra_r_loss_added/FDM_flat_noRloss.png'];
                
                saveas(fig,titleFig,'png');
                titleFig2=['Indra_r_loss_added/FDM_ant_h_','_',...
                    num2str(antHeight(a)), ...
                    '_','Deltaz','_',deltaZstr,'_','Deltax_','_', ...
                    num2str(deltaX),'_distance_source_abslayer',...
                    num2str(maxHinterest-zs),'wedge.pdf'];
                %print (fig, '-dpdf', titleFig2);
                
            end
            
            tx =zs;
            rx = zs;
            part1Title = ['SSA and FDM - wedge '];
            part12Titile = [ ' ','\Deltaz = ',' ',deltaZstr,...
                'm, ','\Deltax = ',' ', ...
                num2str(deltaX),'m, Source height: ', ...
                num2str(antHeight(a)),'m'];
            part21Title = ['Distance from center point of ',...
                'source to beginning of absorption layer: ',...
                num2str(maxHinterest-zs),'m' ];
            part2Title=['Number of points in absorption layer:',...
                ' ', num2str(numPtsAbsoptionLayer)];
            
            plotTitle = {part1Title;part12Titile;part21Title;...
                part2Title};
            
            titleFig =['Indra_r_loss_added/SSA_FDM_ant_h_','_',...
                num2str(antHeight(a)), ...
                '_','Deltaz','_',deltaZstr,'_','Deltax_','_', ...
                num2str(deltaX),'_distance_source_abslayer',...
                num2str(maxHinterest-zs),'wedge2.png'];
            
            filename = 'IndraWedge_results/LPDA-u-kile_E.xls';
            rx_xArr = ones(length(zVectFDM),1).*length(xVect);
            rx_zArr = zVectFDM;
            
            % Vertical comparison:
            pathLossWedge_Indra(Xn,Zn,deltaX,A,tx,rx_xArr,rx_zArr,...
                eFieldTot3,zs,...
                beta,frequency,simulationType,' ',titleFig,filename);
            
            % Comparing with results from Indra (along the surface):
            tx =antHeight;
            rx = antHeight;
            rx_xArr = xVect;
            rx_zArr = ones(length(xVect),1).*antHeight;
            part1Title = ['SSA and FDM - flat surface - along the surface- no additional loss'];
            plotTitle = ' '; 
            titleFig = ['Indra_r_loss_added/SSA_FDM_flat_along_surface_noLossAdded.png'];
            filename = 'IndraWedge2_results/LPDA-u-kile-2_E.xls';
            
            eFieldSSA3=uValuesSplitStep3(ceil(HindexFDM*(antHeight/maxHinterest)),:);
            eFieldFDM3=uValuesFDM3(ceil(HindexFDM*(antHeight/maxHinterest)),:);
            eFieldTot3 = eFieldSSA3;
            eFieldTot3 = vertcat(eFieldTot3,eFieldFDM3);
                
            pathLossIndra_alongX(rx_xArr,rx_zArr,eFieldTot3,frequency,...
                simulationType, plotTitle,titleFig,filename);               
            
            
        end
        
    end
end
