% UpwardsInclinedPlane.m: Perform simulations on a upward inclined
%                           plane, and then compares the results with a
%                           flat plane. The beam propagating along the
%                           inclined plane has the same relative directivty
%                           as the flat plane. The comparison between the
%                           fields are done in the height direction.

clear all
% The inclined plane:
beta =(55/(2*360))*(2*pi); %(3/20)*pi; %  pi/9; % pi/18; % pi/12; %2*beta = half-power beamwidth
A = 10^(0.9);
frequency = 110*10^6;

xDiff =1000; %2569; %1000;% 400; % 1000;
zDiff =20; %250; %  250; % 100; % 250;

theta0 = -asin(zDiff/xDiff); % atan(zDiff/xDiff);

% The delta x value:
deltaX = 1; %1.5;
deltaZ = 1;
maxX = xDiff + 100; %3000; % 1100; %1000;
maxZ = abs(maxX*sin(theta0)); % maxX*tan(theta0);
numPtsAbsoptionLayer = 150;
xDist =xDiff;
%theta0 = -theta0;

% The delta z value:
deltaZarr =[1];% 1.5]; % 1.5;
antHeight = 3; %[2 3 5 10 15 25];
deltaXvect = [1]; % 1.5];

% The irregular terrain:
Xn = [0  maxX];
Zn = [0 maxZ];
maxZcompare = 270;
%theta0 = abs(asin(minZ/maxX));

%plot(Xn,Zn)



[xVect,zSurfaceVect] = interpolate(Xn,Zn,frequency,...
    'linear',deltaX);

%xVect = verticalVector([0:deltaX:maxX]);
maxHinterest = 350+antHeight; % 350 +antHeight;

simulationType = cell(4,1);

%Plot the surface:
surfacePlot = figure();
plot(xVect,zSurfaceVect)
xlabel('Distance [m]');
ylabel('Surface height [m]');
titleFig = ['UpwardsInclinedPlane_results/UpwardsInclinedPlane_surface.png'];
saveas(surfacePlot,titleFig,'png');



% Creating initial field:
[zVectFDM,HindexFDM] =createZvectAbsorptionLayer2(...
    maxHinterest,deltaZ,numPtsAbsoptionLayer);
initialFieldFDM = createInitialField(antHeight,theta0,beta,...
    zVectFDM,A, frequency,'gaussian1');
numZpoints = length(zVectFDM);

% Calculating field:
tic
uValuesSplitStep = SSAirregularTerrainAbsoptionLayer(...
    initialFieldFDM,zVectFDM,...
    xVect,zSurfaceVect,HindexFDM,frequency,...
    numPtsAbsoptionLayer,antHeight);
toc
tic
uValuesFDM = FDMirregularTerrainAbsorptionLayer ...
    (initialFieldFDM,zVectFDM,xVect,zSurfaceVect,...
    HindexFDM,frequency,numPtsAbsoptionLayer);
toc
deltaZstr = num2str(deltaZ);
yText = strcat('Height above the lowest point [m]');

%maxZ = 250;
eFieldSSA  = verticalVector(getVerticalValues(Xn,Zn,xDiff,zDiff,deltaX,deltaZ,...
    xVect,xDist,maxZcompare,uValuesSplitStep,'up'))';

eFieldFDM = verticalVector(getVerticalValues(Xn,Zn,xDiff,zDiff,deltaX,deltaZ,...
    xVect,xDist,maxZcompare,uValuesFDM,'up'))';


% xCoord =  find(xVect >= xDist ,1);
% eFieldSSA = zeros(1,length(zVectFDM));
% eFieldFDM = zeros(1,length(zVectFDM));
% for i = 1:length(zVectFDM)
%     eFieldSSA(1,i) = uValuesSplitStep(i,xCoord);
%     eFieldFDM(1,i) = uValuesFDM(i,xCoord);
% end
simulationType{1,1} = ['SSA inclined'];

simulationType{2,1} = ['FDM inclined'];
tx = antHeight;
rx = antHeight;

eFieldTot = eFieldSSA;
eFieldTot = vertcat(eFieldTot,eFieldFDM);



minPlotLevel = 10^(-7);
for i = 1:1
    % Plot SSA figuure:
     fig2 = figure('visible','off');
   
    plotScale =length(zVectFDM); % ceil(2*length(zVectFDM)/3)
   
    part1Title = ['Split-Step Algorithm - flat surface'];
    titleVal2 = {part1Title};
    tic
    uValuesAux = uValuesSplitStep(1:plotScale,:);
    uValuesAux(abs(uValuesAux)<minPlotLevel) = minPlotLevel;
    minVal = 10.*log10(min(min(abs(uValuesAux).^2)));
    maxVal = 10.*log10(max(max(abs(uValuesAux).^2)));
    disp('contourf is on')
    contourf(xVect,zVectFDM(1:plotScale,1),...
        10.*log10(abs(uValuesAux.^2)),50)
    hold on
    disp('contour is on')
    contour(xVect,zVectFDM(1:plotScale,1),...
        10.*log10(abs(uValuesAux.^2)),50)
    hold on
     eFieldSSA  = verticalVector(getVerticalValues(Xn,Zn,xDiff,zDiff,deltaX,deltaZ,...
    xVect,xDist,maxZcompare,uValuesSplitStep,'up'))';
    
    %title(titleVal2)
    xlabel('Distance [m]');
    ylabel(yText);
    grid on
    
    titleFig = ['UpwardsInclinedPlane_results/SSA_field_InclUp.png'];
    disp('saving ...')
    saveas(fig2,titleFig,'png');
    titleFig = titleFig(1:(length(titleFig)-4));
    titleFig = horzcat(titleFig,'.pdf');
    save2pdf(titleFig);
    toc
    
    tic
    % Plot FDM figure:
    fig = figure('visible','off');
    part1Title = ['Finite-Difference Method - flat surface'];
    titleVal2 = {part1Title};
   
    uValuesAux = uValuesFDM(1:plotScale,:);
    uValuesAux(abs(uValuesAux)<minPlotLevel) = minPlotLevel;
    minVal = 10.*log10(min(min(abs(uValuesAux).^2)));
    maxVal = 10.*log10(max(max(abs(uValuesAux).^2)));
    contourf(xVect,zVectFDM(1:plotScale,1),...
        10.*log10(abs(uValuesAux.^2)),50)
    hold on
    contour(xVect,zVectFDM(1:plotScale,1),...
        10.*log10(abs(uValuesAux.^2)),50)
    %title(titleVal2)
    hold on
    eFieldFDM = verticalVector(getVerticalValues(Xn,Zn,xDiff,zDiff,deltaX,deltaZ,...
    xVect,xDist,maxZcompare,uValuesFDM,'up'))';
    
    xlabel('Distance [m]');
    ylabel(yText);
    grid on
    titleFig = ['UpwardsInclinedPlane_results/FDM_field_InclUp.png'];
    saveas(fig,titleFig,'png');
    titleFig = titleFig(1:(length(titleFig)-4));
    titleFig = horzcat(titleFig,'.pdf');
    save2pdf(titleFig);
    toc
    
end % Plot figure, not in use

%Comparing with results from Indra (along the surface):
tx =antHeight;
rx = antHeight;
rx_xArr = xVect;
rx_zArr = ones(length(xVect),1).*antHeight;
%part1Title = ['SSA and FDM - flat surface - along the surface'];
plotTitle = ' '; %{part1Title};
titleFig = ['UpwardsInclinedPlane_results/SSA_FDM_InclUp.png'];
filename = 'IndraWedge2_results/LPDA-u-kile-2_E.xls';


rx_xArr = ones(length(zVectFDM),1).*length(xVect);
rx_zArr = zVectFDM;
% pathLossIndra_alongX(rx_xArr,rx_zArr,eFieldTot,frequency,...
%     simulationType, plotTitle,titleFig,filename);

simulationType1 = cell(2,1);
simulationType1{1,1} = 'SSA inclined';
simulationType1{2,1} = 'FDM inclined';
%%
pathLossWedge_Indra(Xn,Zn,deltaX,A,tx,rx_xArr,rx_zArr,eFieldTot,...
    antHeight,beta,frequency,simulationType1, plotTitle,titleFig,' ')

%%
% Calculating the field along the flat surface
theta0 = 0;
Xn_flat = [0 xDist]; %3000]; % xDist];
Zn_flat = [0 0];


% Creating initial field:
[zVectFDM,HindexFDM] =createZvectAbsorptionLayer2(...
    maxHinterest,deltaZ,numPtsAbsoptionLayer);
initialFieldFDM = createInitialField(antHeight,theta0,beta,...
    zVectFDM,A, frequency,'gaussian1');
numZpoints = length(zVectFDM);
%
% [xVect,zSurfaceVect_flat] = interpolate(Xn_flat,Zn_flat,frequency,...
%     'linear',deltaX);
% 
% [zVectFDM,HindexFDM] =createZvectAbsorptionLayer2(...
%     maxHinterest,deltaZ,numPtsAbsoptionLayer);
% initialFieldFDM = createInitialField(antHeight,theta0,beta,...
%     zVectFDM,A, frequency,'gaussian1');
% numZpoints = length(zVectFDM);

% Calculating field:

% tic
% uValuesSplitStep_flat = SSAirregularTerrainAbsoptionLayer(...
%     initialFieldFDM,zVectFDM,...
%     xVect,zSurfaceVect_flat,HindexFDM,frequency,...
%     numPtsAbsoptionLayer,antHeight);
% toc
% tic
% uValuesFDMalne_flat = FDMirregularTerrainAbsorptionLayer ...
%     (initialFieldFDM,zVectFDM,xVect,zSurfaceVect_flat,...
%     HindexFDM,frequency,numPtsAbsoptionLayer);
% toc

tic
uValuesSplitStep_flat =splitStepAlgorithmAbsorptionLayer(...
    initialFieldFDM,zVectFDM,xVect, ...
    HindexFDM,frequency,numPtsAbsoptionLayer);
toc
tic
[uValuesFDMalne_flat,maxEigVal]=...
    FDMAbsorptionLayerNumEfficient2(initialFieldFDM, ...
    zVectFDM,xVect,HindexFDM,frequency,numPtsAbsoptionLayer);
toc


clear eFieldSSA;
clear eFieldFDM;
%xIndex = find(xVect >= 1000,1);

eFieldSSA = zeros(1,length(eFieldTot(1,:))); % length(zVectFDM));
eFieldFDM = zeros(1,length(eFieldTot(1,:))); % length(zVectFDM));
for i = 1:length(eFieldSSA) % length(zVectFDM)
    eFieldSSA(1,i) = uValuesSplitStep_flat(i,...
        length(xVect));
    eFieldFDM(1,i) = uValuesFDMalne_flat(i,...
        length(xVect));
    
end


simulationType{3,1} = ['SSA flat'];
simulationType{4,1} = ['FDM flat'];
eFieldTot = vertcat(eFieldTot,eFieldSSA);
eFieldTot = vertcat(eFieldTot,eFieldFDM);

%Comparing with results from Indra (along the surface):
tx =antHeight;
rx = antHeight;
rx_xArr = ones(length(zVectFDM),1).*length(xVect);
rx_zArr = zVectFDM;
part1Title = ' ';%['SSA and FDM - flat surface - along the surface'];
plotTitle = ' '; % {part1Title};
%%
titleFig = ['UpwardsInclinedPlane_results/SSA_FDM_compare_InclUp.png'];
filename = 'IndraWedge_results/LPDA-u-kile_E.xls';
numCases = 4;
startIndex = 4;
numElts = length(eFieldTot(1,:)); % length(zVectFDM);
eFieldTot1 = zeros(numCases,length(eFieldTot(1,startIndex:numElts)));
for i = 1:numCases
    eFieldTot1(i,:) = eFieldTot(i,startIndex:numElts);
end

pathLossWedge_Indra(Xn,Zn,deltaX,A,tx,rx_xArr,rx_zArr,eFieldTot1,...
    antHeight,beta,frequency,simulationType, plotTitle,titleFig,filename)
%%
compareHeight = 50;
 titleFig3 =['UpwardsInclinedPlane_results/',...
     'SSA_FDM_flat_height_varying_InclUp_zoomed1.png']; 
  pathLossFlat_Indra_minComp(rx_xArr,rx_zArr,eFieldTot1,...
     frequency,simulationType, plotTitle,titleFig3,filename,compareHeight)
 %%
 titleFig4 =['UpwardsInclinedPlane_results/',...
     'SSA_FDM_flat_height_varying_InclUp_zoomed2.png'];
  pathLossFlat_Indra(rx_xArr,rx_zArr,eFieldTot1,...
     frequency,simulationType, plotTitle,titleFig4,filename,compareHeight)
%%
 titleFig5 =['UpwardsInclinedPlane_results/',...
     'SSA_FDM_flat_height_varying_InclUp_pathLoss.png'];
 
 compareHeight2 = 250;
 pathLossWedge_Hviid(antHeight,rx_xArr,rx_zArr,eFieldTot1,...
    frequency,simulationType, plotTitle,titleFig5,' ',compareHeight2)
%%
% Plot SSA figuure:
yText = strcat('Height above surface [m]');
fig2 = figure('visible','off');
part1Title = ['Split-Step Algorithm - flat surface'];
titleVal2 = {part1Title};
minValuePlot = 10^-8;
uValuesAux = uValuesSplitStep_flat;
uValuesAux(abs(uValuesAux)<minValuePlot) = minValuePlot;
contourf(xVect,zVectFDM,10.*log10(abs(uValuesAux.^2)),50)
hold on
contour(xVect,zVectFDM,10.*log10(abs(uValuesAux.^2)),50)
%title(titleVal2)
xlabel('Distance [m]');
ylabel(yText);
grid on
titleFig = ['UpwardsInclinedPlane_results/SSA_flat_InclUp.png'];
saveas(fig2,titleFig,'png');

% Plot FDM figure:
fig = figure('visible','off');
part1Title = ['Finite-Difference Method - flat surface'];
titleVal2 = {part1Title};

uValuesAux = uValuesFDMalne_flat;
uValuesAux(abs(uValuesAux)<minValuePlot) =minValuePlot;
contourf(xVect,zVectFDM,10.*log10(abs(uValuesAux.^2)),50)
hold on
contour(xVect,zVectFDM,10.*log10(abs(uValuesAux.^2)),50)
%title(titleVal2)
xlabel('Distance [m]');
ylabel(yText);
grid on
titleFig = ['UpwardsInclinedPlane_results/FDM_flat_InclUp.png'];
saveas(fig,titleFig,'png');

%%
% Compare along the surface at constant height of 40 m above the lowest
% point:

%eFieldAlongTot 
height = 40;
startIndex = 101; %100;
eSSArunway = (uValuesSplitStep(height,startIndex:length(xVect)));
eFieldAlongTot  =  eSSArunway;
eFDMrunway = (uValuesFDM(height,startIndex:length(xVect)));
eFieldAlongTot = vertcat(eFieldAlongTot,eFDMrunway);

eSSAflat = (uValuesSplitStep_flat(height,startIndex:length(xVect)));
eFieldAlongTot = vertcat(eFieldAlongTot,eSSAflat);
eFDMflat = (uValuesFDMalne_flat(height,startIndex:length(xVect)));
eFieldAlongTot = vertcat(eFieldAlongTot,eFDMflat);

imulationType2 = cell(4,1);
simulationType2{1,1} = ['SSA - upwards'];
simulationType2{2,1} = ['FDM - upwards'];
simulationType2{3,1} = ['SSA - flat'];
simulationType2{4,1} = ['FDM - flat'];

rx_xArr3 = xVect(startIndex:length(xVect));
rx_zArr3 =  ones(length(rx_xArr3),1).*height;
titleFig3 = ['UpwardsInclinedPlane_results/',...
    'SSA_FDM_pathloss_horizontal_InclUp.png'];
antHeight2 = antHeight - zSurfaceVect(1);

% pathLossWedge_Hviid(antHeight2,rx_zArr3,rx_xArr3,eFieldAlongTot,...
%     frequency,simulationType2, ' ',titleFig3,' ',0)
titleFig4 = ['UpwardsInclinedPlane_results/',...
    'SSA_FDM_pathloss_horizontal2_InclUp.png'];

pathLossIndra_alongX(rx_xArr3,rx_zArr3,eFieldAlongTot,...
    frequency,simulationType2,' ',titleFig4,' ')


% Comparing fields along the surface at constant height above the surface
height2 = 15;
startIndex2 = 101;
fieldVect2 = [ceil(startIndex2/deltaX):length(xVect)/deltaX];
eSSArunway2 = zeros(length(fieldVect2),1);
eFDMrunway2 = zeros(length(fieldVect2),1);
for i = 1:length(fieldVect2)
    %a = zSurfaceNorm(i);
    eSSArunway2(i,1) = uValuesSplitStep(round(zSurfaceVect(i)+height), ...
        fieldVect2(i));
    eFDMrunway2(i,1) = uValuesFDM(round(zSurfaceVect(i)+height), ...
        fieldVect2(i));
end
%%
close all

eFieldAlongTot2  =  eSSArunway2';
eFieldAlongTot2 = vertcat(eFieldAlongTot2,eFDMrunway2');
eSSAflat2 = (uValuesSplitStep_flat(height,startIndex2:length(xVect)));
eFieldAlongTot2 = vertcat(eFieldAlongTot2,eSSAflat2);
eFDMflat2 = (uValuesFDMalne_flat(height,startIndex2:length(xVect)));
eFieldAlongTot2 = vertcat(eFieldAlongTot2,eFDMflat2);
% eFieldAlongTot2 = vertcat(eFieldAlongTot2,zSurfaceNorm);

% simulationType3 = cell(4,1);
% simulationType3{1,1} = ['SSA - runway'];
% simulationType3{2,1} = ['FDM - runway'];
% simulationType3{3,1} = ['SSA - flat'];
% simulationType3{4,1} = ['FDM - flat'];
% simulationType3{5,1} = ['Runway '];

titleFig5 = ['UpwardsInclinedPlane_results/',...
    'SSA_FDM_pathloss_horizontal_cst_diff_surface_InclUp.png'];
rx_xArr4 = xVect(startIndex2:length(xVect));
rx_zArr4 =  ones(length(rx_xArr4),1).*height;


pathLossIndra_alongX(rx_xArr4,rx_zArr4,eFieldAlongTot2,...
    frequency,simulationType2,' ',titleFig5,' ')
%get(gcf,'CurrentAxes')
ax1 = gca;


fig = figure();
% ax2 = gca;
%  ax2 = axes(... %'Position',get(ax1,'Position'),...
%            'XAxisLocation','top',...
%            'YAxisLocation','right',...
%            'Color','none',...
%            'XColor','k','YColor','k');
% hold(axes_handle,'on')
% plot((xVect(startIndex2:length(xVect))),zSurfaceNorm...
%     (startIndex2:length(xVect)),'k','Parent',ax2);      
 semilogx((xVect(startIndex2:length(xVect))),zSurfaceVect...
     (startIndex2:length(xVect))); %,'Parent',ax2);
% linkaxes([ax1 ax2],'y');
% linkaxes([ax1 ax2],'x');
legend('Runway surface profile','Location','SouthEast');
xlabel('Distance [m]');
ylabel('Height [m]');
plotwidth = 560;
plotheight = 200;
set(fig, 'Position', [500 100 plotwidth plotheight]);
grid on
titleFig6 = ['UpwardsInclinedPlane_results/',...
    'Runway_profile_small_InclUp.png'];
saveas(fig,titleFig,'png');
titleFig6 = titleFig6(1:(length(titleFig6)-4));
titleFig6 = horzcat(titleFig6,'.pdf');
%print (fig, '-dpdf', titleFig);
save2pdf(titleFig6);





