% WedgeComparison_Hviid.m: Simulates the field over the wedge given in the
%                          Hviid article using the SSA and FDM, and 
%                          compares with the results over a flat surface.
                         

clear all
% The inclined plane:
beta = pi/18; 
A = 1; 
frequency = 100*10^6;
theta0 = 0;

% The delta x and delta z value:
deltaX = 1;
deltaZ = 1;

antHeight = 3; 
maxX = 5000;
halfWay = maxX/2;
maxZ = 50;
numPtsAbsoptionLayer = 250; 

% The irregular terrain:
Xn = [0 halfWay maxX];
Zn = [0 maxZ 0];
maxZcompare = 500;

% Interpolating the surface:
[xVect,zSurfaceVect] = interpolate(Xn,Zn,frequency,...
    'linear',deltaX);

maxHinterest = 650+antHeight; 

simulationType = cell(4,1);

%Plot the surface:
surfacePlot = figure();
plot(xVect,zSurfaceVect,'k')
xlabel('Distance [m]');
ylabel('Surface height [m]');
plotwidth = 560;
plotheight = 200;
set(surfacePlot, 'Position', [500 100 plotwidth plotheight]);
titleFig = ['WedgeComparison_Hviid_Results/WedgeHviid_surface.png'];
saveas(surfacePlot,titleFig,'png');
titleFig = titleFig(1:(length(titleFig)-4));
titleFig = horzcat(titleFig,'.pdf');
save2pdf(titleFig);



% Creating initial field:
[zVectFDM,HindexFDM] =createZvectAbsorptionLayer2(...
    maxHinterest,deltaZ,numPtsAbsoptionLayer);
initialFieldFDM = createInitialField(antHeight,theta0,beta,...
    zVectFDM,A, frequency,'gaussian1');
numZpoints = length(zVectFDM);

% Calculating field:
tic
uValuesSplitStep = SSAirregularTerrainAbsoptionLayer(...
    initialFieldFDM,zVectFDM,...
    xVect,zSurfaceVect,HindexFDM,frequency,...
    numPtsAbsoptionLayer,antHeight);
toc
tic
uValuesFDM = FDMirregularTerrainAbsorptionLayer ...
    (initialFieldFDM,zVectFDM,xVect,zSurfaceVect,...
    HindexFDM,frequency,numPtsAbsoptionLayer);
toc
deltaZstr = num2str(deltaZ);
yText = strcat('Height above the lowest point [m]');


% Extracting the field at distance of interest:
 xCoord =  find(xVect >= maxX ,1);
 eFieldSSA = zeros(1,((maxZcompare +1)/deltaZ));
 eFieldFDM = zeros(1,((maxZcompare +1)/deltaZ));
 for i = 1:(maxZcompare +1)
     eFieldSSA(1,i) = uValuesSplitStep(i,xCoord);
     eFieldFDM(1,i) = uValuesFDM(i,xCoord);
 end
simulationType{1,1} = ['SSA wedge'];

simulationType{2,1} = ['FDM wedge'];
tx = antHeight;
rx = antHeight;

eFieldTot = eFieldSSA;
eFieldTot = vertcat(eFieldTot,eFieldFDM);




minPlotLevel = 10^(-7);
for i = 1:1
    % Plot SSA figuure:
     fig2 = figure('visible','off');
   
    plotScale =length(zVectFDM); 
   
    part1Title = ['Split-Step Algorithm - flat surface'];
    titleVal2 = {part1Title};
    tic
    uValuesAux = uValuesSplitStep(1:plotScale,:);
    uValuesAux(abs(uValuesAux)<minPlotLevel) = minPlotLevel;
    minVal = 10.*log10(min(min(abs(uValuesAux).^2)));
    maxVal = 10.*log10(max(max(abs(uValuesAux).^2)));
    disp('contourf is on')
    contourf(xVect,zVectFDM(1:plotScale,1),...
        10.*log10(abs(uValuesAux.^2)),50)
    hold on
    disp('contour is on')
    contour(xVect,zVectFDM(1:plotScale,1),...
        10.*log10(abs(uValuesAux.^2)),50)
    hold on
   
    %title(titleVal2)
    xlabel('Distance [m]');
    ylabel(yText);
    grid on
    
    titleFig = ['WedgeComparison_Hviid_Results/SSA_field_freq_',...
        num2str(frequency/(10^6)),'_HviidWedge.png'];
    disp('saving ...')
    saveas(fig2,titleFig,'png');
    titleFig = titleFig(1:(length(titleFig)-4));
    titleFig = horzcat(titleFig,'.pdf');
    save2pdf(titleFig);
    toc
    
    tic
    % Plot FDM figure:
    fig = figure('visible','off');
    part1Title = ['Finite-Difference Method - flat surface'];
    titleVal2 = {part1Title};
   
    uValuesAux = uValuesFDM(1:plotScale,:);
    uValuesAux(abs(uValuesAux)<minPlotLevel) = minPlotLevel;
    minVal = 10.*log10(min(min(abs(uValuesAux).^2)));
    maxVal = 10.*log10(max(max(abs(uValuesAux).^2)));
    contourf(xVect,zVectFDM(1:plotScale,1),...
        10.*log10(abs(uValuesAux.^2)),50)
    hold on
    contour(xVect,zVectFDM(1:plotScale,1),...
        10.*log10(abs(uValuesAux.^2)),50)
    %title(titleVal2)
    hold on
    xlabel('Distance [m]');
    ylabel(yText);
    grid on
    titleFig = ['WedgeComparison_Hviid_Results/FDM_field_freq_',...
        num2str(frequency/(10^6)),'_HviidWedge.png'];
    saveas(fig,titleFig,'png');
    titleFig = titleFig(1:(length(titleFig)-4));
    titleFig = horzcat(titleFig,'.pdf');
    save2pdf(titleFig);
    toc
    
end 

% Zoomed plots:
for i = 1:1
    % Plot SSA figuure:
     fig2 = figure('visible','off');
   
    plotScale =ceil(0.35*length(zVectFDM)); % ceil(2*length(zVectFDM)/3)
   
    part1Title = ['Split-Step Algorithm - flat surface'];
    titleVal2 = {part1Title};
    tic
    uValuesAux = uValuesSplitStep(1:plotScale,:);
    uValuesAux(abs(uValuesAux)<minPlotLevel) = minPlotLevel;
    minVal = 10.*log10(min(min(abs(uValuesAux).^2)));
    maxVal = 10.*log10(max(max(abs(uValuesAux).^2)));
    disp('contourf is on')
    contourf(xVect,zVectFDM(1:plotScale,1),...
        10.*log10(abs(uValuesAux.^2)),50)
    hold on
    disp('contour is on')
    contour(xVect,zVectFDM(1:plotScale,1),...
        10.*log10(abs(uValuesAux.^2)),50)
    hold on
   
    %title(titleVal2)
    xlabel('Distance [m]');
    ylabel(yText);
    grid on
    
    titleFig = ['WedgeComparison_Hviid_Results/SSA_field_freq_',...
        num2str(frequency/(10^6)),'_HviidWedge_zoomed.png'];
    disp('saving ...')
    saveas(fig2,titleFig,'png');
    titleFig = titleFig(1:(length(titleFig)-4));
    titleFig = horzcat(titleFig,'.pdf');
    save2pdf(titleFig);
    toc
    
    tic
    % Plot FDM figure:
    fig = figure('visible','off');
    part1Title = ['Finite-Difference Method - flat surface'];
    titleVal2 = {part1Title};
   
    uValuesAux = uValuesFDM(1:plotScale,:);
    uValuesAux(abs(uValuesAux)<minPlotLevel) = minPlotLevel;
    minVal = 10.*log10(min(min(abs(uValuesAux).^2)));
    maxVal = 10.*log10(max(max(abs(uValuesAux).^2)));
    contourf(xVect,zVectFDM(1:plotScale,1),...
        10.*log10(abs(uValuesAux.^2)),50)
    hold on
    contour(xVect,zVectFDM(1:plotScale,1),...
        10.*log10(abs(uValuesAux.^2)),50)
    %title(titleVal2)
    hold on
    xlabel('Distance [m]');
    ylabel(yText);
    grid on
    titleFig = ['WedgeComparison_Hviid_Results/FDM_field_freq_',...
        num2str(frequency/(10^6)),'_HviidWedge_zoomed.png'];
    saveas(fig,titleFig,'png');
    titleFig = titleFig(1:(length(titleFig)-4));
    titleFig = horzcat(titleFig,'.pdf');
    save2pdf(titleFig);
    toc
    
end 

% Comparing with results from Indra (along the surface):
tx =antHeight;
rx = antHeight;
rx_xArr = xVect;
rx_zArr = ones(length(xVect),1).*antHeight;
plotTitle = ' '; 
titleFig = ['WedgeComparison_Hviid_Results/SSA_FDM_freq_',...
        num2str(frequency/(10^6)),'_HviidWedge.png'];
filename = 'IndraWedge2_results/LPDA-u-kile-2_E.xls';

rx_xArr = ones(length(zVectFDM),1).*length(xVect);
rx_zArr = zVectFDM;

simulationType1 = cell(2,1);
simulationType1{1,1} = 'SSA';
simulationType1{2,1} = 'FDM';
compareHeight_Hviid  = 250;
close all
initCompare = initialFieldFDM(1:maxZcompare +1);
eTest = zeros(4,length(initCompare));

for i = 1:4
    eTest(1,:) = initCompare'./eFieldTot(1,:);
end

pathLossWedge_Hviid(antHeight,rx_xArr,rx_zArr,eFieldTot,...
    frequency,simulationType1, plotTitle,titleFig,' ',compareHeight_Hviid)

% Calculating the field along the flat surface
theta0 = 0;
Xn_flat = [0 maxX]; 
Zn_flat = [0 0];

% Creating the field (flat surface);
[zVectFDM,HindexFDM] =createZvectAbsorptionLayer2(...
    maxHinterest,deltaZ,numPtsAbsoptionLayer);
initialFieldFDM = createInitialField(antHeight,theta0,beta,...
    zVectFDM,A, frequency,'gaussian1');
numZpoints = length(zVectFDM);

% Calculating the fields (flat surface):
tic
uValuesSplitStepFlat =splitStepAlgorithmAbsorptionLayer(...
    initialFieldFDM,zVectFDM,xVect, ...
    HindexFDM,frequency,numPtsAbsoptionLayer);
toc
tic
[uValuesFDMalneFlat,maxEigVal]=...
    FDMAbsorptionLayerNumEfficient2(initialFieldFDM, ...
    zVectFDM,xVect,HindexFDM,frequency,numPtsAbsoptionLayer);
toc

clear eFieldSSA;
clear eFieldFDM;

% Extracting the field for vertical comparison:
eFieldSSA = zeros(1,length(eFieldTot(1,:))); 
eFieldFDM = zeros(1,length(eFieldTot(1,:)));
for i = 1:length(eFieldSSA) 
    eFieldSSA(1,i) = uValuesSplitStepFlat(i,...
        length(xVect));
    eFieldFDM(1,i) = uValuesFDMalneFlat(i,...
        length(xVect));
    
end

simulationType{3,1} = ['SSA flat'];
simulationType{4,1} = ['FDM flat'];
eFieldTot = vertcat(eFieldTot,eFieldSSA);
eFieldTot = vertcat(eFieldTot,eFieldFDM);

% Comparing with results from Indra, vertical comparison:
tx =antHeight;
rx = antHeight;
rx_xArr = ones(length(zVectFDM),1).*length(xVect);
rx_zArr = zVectFDM;
part1Title = ' ';%['SSA and FDM - flat surface - along the surface'];
plotTitle = ' '; % {part1Title};

titleFig = [...
    'WedgeComparison_Hviid_Results/SSA_FDM_compareFlat_HviidWedge.png'];
filename = 'IndraWedge_results/LPDA-u-kile_E.xls';
numCases = 4;
startIndex = 4;
numElts = length(eFieldTot(1,:)); 
eFieldTot1 =eFieldTot;

compareHeight = 500; % maximum height of comparison
pathLossWedge_Hviid(antHeight,rx_xArr,rx_zArr,eFieldTot1,...
    frequency,simulationType, plotTitle,titleFig,' ',compareHeight)

compareHeight = 50; % new maximum height of comparison
 titleFig3 =['WedgeComparison_Hviid_Results/',...
     'SSA_FDM_flat_height_varying_HviidWedge_zoomed1.png']; 
  pathLossFlat_Indra_minComp(rx_xArr,rx_zArr,eFieldTot1,...
     frequency,simulationType, plotTitle,titleFig3,filename,compareHeight)
 
 titleFig4 =['WedgeComparison_Hviid_Results/',...
     'SSA_FDM_flat_height_varying_HviidWedge_zoomed2.png'];
  pathLossFlat_Indra(rx_xArr,rx_zArr,eFieldTot1,...
     frequency,simulationType, plotTitle,titleFig4,filename,compareHeight)
 
 % Comparing along the surface, horizontal comparison:

simulationType2 = cell(4,1);
simulationType2{1,1} = ['SSA - wedge'];
simulationType2{2,1} = ['FDM - wedge'];
simulationType2{3,1} = ['SSA - flat'];
simulationType2{4,1} = ['FDM - flat'];
%eFieldAlongTot 
height = 65;

startIndex = 121; 
eSSAwedge = (uValuesSplitStep(height,startIndex:length(xVect)));
eFieldAlongTot  =  eSSAwedge;
eFDMwedge = (uValuesFDM(height,startIndex:length(xVect)));
eFieldAlongTot = vertcat(eFieldAlongTot,eFDMwedge);

eSSAflat = (uValuesSplitStepFlat(height,startIndex:length(xVect)));
eFieldAlongTot = vertcat(eFieldAlongTot,eSSAflat);
eFDMflat = (uValuesFDMalneFlat(height,startIndex:length(xVect)));
eFieldAlongTot = vertcat(eFieldAlongTot,eFDMflat);

rx_xArr3 = xVect(startIndex:length(xVect));
rx_zArr3 =  ones(length(rx_xArr3),1).*height;
titleFig3 = ['WedgeComparison_Hviid_Results/',...
    'SSA_FDM_pathloss_horizontal_HviidWedge.png'];

pathLossWedge_Hviid(antHeight,rx_zArr3,rx_xArr3,eFieldAlongTot,...
    frequency,simulationType2, ' ',titleFig3,' ',0)
titleFig4 = ['WedgeComparison_Hviid_Results/',...
    'SSA_FDM_pathloss_horizontal2_HviidWedge.png'];

pathLossIndra_alongX(rx_xArr3,rx_zArr3,eFieldAlongTot,...
    frequency,simulationType2,' ',titleFig4,' ')


% Comparing fields along the surface at constant height above the surface
height2 = 15;
startIndex2 = 121;
fieldVect2 = [ceil(startIndex2/deltaX):length(xVect)/deltaX];
eSSAwedge2 = zeros(length(fieldVect2),1);
eFDMwedge2 = zeros(length(fieldVect2),1);
for i = 1:length(fieldVect2)
    eSSAwedge2(i,1) = uValuesSplitStep(round(zSurfaceVect(i)+height), ...
        fieldVect2(i));
    eFDMwedge2(i,1) = uValuesFDM(round(zSurfaceVect(i)+height), ...
        fieldVect2(i));
end

eFieldAlongTot2  =  eSSAwedge2';
eFieldAlongTot2 = vertcat(eFieldAlongTot2,eFDMwedge2');
eSSAflat2 = (uValuesSplitStepFlat(height,startIndex2:length(xVect)));
eFieldAlongTot2 = vertcat(eFieldAlongTot2,eSSAflat2);
eFDMflat2 = (uValuesFDMalneFlat(height,startIndex2:length(xVect)));
eFieldAlongTot2 = vertcat(eFieldAlongTot2,eFDMflat2);

titleFig5 = ['WedgeComparison_Hviid_Results/',...
    'SSA_FDM_pathloss_horizontal_cst_diff_surface_HviidWedge.png'];
rx_xArr4 = xVect(startIndex2:length(xVect));
rx_zArr4 =  ones(length(rx_xArr4),1).*height;


pathLossIndra_alongX(rx_xArr4,rx_zArr4,eFieldAlongTot2,...
    frequency,simulationType2,' ',titleFig5,' ')
%get(gcf,'CurrentAxes')
ax1 = gca;

% Plotting the wedge surface with logarithmic scale:
fig = figure();

semilogx((xVect(startIndex2:length(xVect))),zSurfaceVect...
     (startIndex2:length(xVect))); 

legend('Wedge surface','Location','NorthWest');
xlabel('Distance [m]');
ylabel('Height [m]');
plotwidth = 560;
plotheight = 200;
set(fig, 'Position', [500 100 plotwidth plotheight]);
grid on
titleFig6 = ['WedgeComparison_Hviid_Results/',...
    'Runway_profile_small_HviidWedge.png'];
saveas(fig,titleFig,'png');
titleFig6 = titleFig6(1:(length(titleFig6)-4));
titleFig6 = horzcat(titleFig6,'.pdf');
%print (fig, '-dpdf', titleFig);
save2pdf(titleFig6);
