% createAbsorptionLayer.m: Creates an absorption layer for the given
%                          parameters.
% intitialField: The initial field.
% maxHeigthInterestZIndex: The index of the maximum height of interest.
% numPointsInLayer: Number of points in the absorption layer

% return: indexOfRefraction: Vector containing the index of refraction in
%                            the entire z-direction

function [indexOfRefraction] = createAbsorptionLayer(intitialField,...
    maxHeigthInterestZIndex,numPointsInLayer)
numZpoints = length(intitialField);
Hindex = maxHeigthInterestZIndex;

indiceValues = verticalVector([1:1:(numZpoints-Hindex+1)]);
indiceValues = (indiceValues./(numPointsInLayer));
gamma0 = 0.15*0.098;
absorptionLayer = gamma0.*(indiceValues);

indexOfRefraction = ones(numZpoints,1);
indexOfRefraction(Hindex:numZpoints,1) = ...
    indexOfRefraction(Hindex:numZpoints,1) + 1j.*absorptionLayer;

end


