% initialField.m: Creates the initial field, using the specified beam
%                 shape.
% zs: source height in meters
% theta0: elevation angle in radians
% beta: Half-power beamwidth (Gaussian beam)
% zVect: Vector containing the heights for field estimation in z-direction
% A: parameter for calculating free-space loss
% frequency: the frequency of operation
% source: 'gaussian1', 'gaussian2' or 'circular'
% return: initialField: The initial field for range x=0


function initialField = createInitialField(zs,theta0,beta,zVect,A,...
    frequency, source)
c = 3*10^8;
lambda = c/frequency;
k = 2*pi/lambda;


numZpoints = length(zVect);
initialField = zeros(numZpoints,1);

switch source
    case 'gaussian1'
        initialField = A.*(k.*beta./(2.*sqrt(2.*log10(2)))) ...
            .*exp(-1j.*k.*theta0.*zVect)...
            .*exp(-((beta.^2)./(8.*log10(2))).*(k.^2).*((zVect-zs).^2));
        
    case 'gaussian2'
        theta1 = beta;
        theta2 = theta0;
        initialField = sqrt(k).*tan(theta1).*exp(-((k.^2)./2).*((zVect-zs).^2).* ...
            (tan(theta1).^2)).*exp(1j.*k.*(zVect-zs).*sin(theta2));
    case 'circular'
        auxTheta = asin(abs((zVect-zs))./A);
%         for x = 1:length(auxTheta)
%             if (auxTheta(x) > pi/2)
%                 auxTheta(x) = 0;
%             end
%         end
        auxTheta(abs(auxTheta) > (pi/4)) =pi/2;
        %auxTheta(auxTheta < (-pi/2)) =0; %pi;
        initialField = A*cos(auxTheta);
        
end
initialField(1) = 0;        
end

