% createZvectAbsorptionLayer.m: Calculates the z-vector based on the maximum
%                            heigth of interest.
% Usage: To be used in combination with the split-step algorithm and
%        finite-difference method where an absorption is included.
 
% maxHeigthInterest: The maximum heigth of interest
% deltaZ: The spacing between the elements in the z-vector

% return:
%         zVect: Vector containing the z-values
%         Hindex: The index of the value closest to the maximum height of 
%                 interest in the zVect.


function [zVect, Hindex] = createZvectAbsorptionLayer(maxHeigthInterest,deltaZ)
L = 2*ceil(maxHeigthInterest/deltaZ);
%L =ceil(maxHeigthInterest/deltaZ);
zVect = verticalVector([0:deltaZ:L]);

% Finding the closest index to the height of interest:
Hindex = ceil(((L/deltaZ) +1)*(maxHeigthInterest/L));
end

