% createZvectAbsorptionLayer2.m: Calculates the z-vector based on the 
%                                maximum heigth of interest. Includes
%                                additional height for absorption layer.
% Usage: To be used in combination with the split-step algorithm and
%        finite-difference method where an absorption is included.
 
% maxHeigthInterest: The maximum heigth of interest
% deltaZ: The spacing between the elements in the z-vector
% numPointsInAbsorptionLayer: The number of points in the absorption layer

% return:
%         zVect: Vector containing the z-values
%         Hindex: The index of the value closest to the maximum height of 
%                 interest in the zVect.


function [zVect, Hindex]=createZvectAbsorptionLayer2(maxHeigthInterest, ...
    deltaZ,numPointsInAbsorptionLayer)
L = ceil((maxHeigthInterest/deltaZ))+ (numPointsInAbsorptionLayer);
zVect = verticalVector([0:deltaZ:L]);

% Finding the closest index to the height of interest:
Hindex = ceil(((L/deltaZ) +1)*(maxHeigthInterest/L));
end

