% discreteMatchedFunction.m: Computes the discrete matched function for a
%                            given set of u-values.

% uIn: vector containing the input u-values at a given range along the
%      z-axis
% deltaZ: the distance beween two points at the z-axis.
% alpha: surface property (perfect reflector: alpha =0)

% return: gVect: vector containing the result.

% NB: length(gVect) = length(uIn)-2

function [gVect] = discreteMatchedFunction(uIn,deltaZ,alpha)

% Implemented using ''algorithm'' proposed in Levy, p.154
uIn = verticalVector(uIn);
numPoints = length(uIn);

gVect = ((uIn(3:numPoints,1)-uIn(1:(numPoints-2),1))./(2.*deltaZ)) + ...
    (alpha.*uIn(2:(numPoints-1),1));

end


