% finiteDifferenceMethod.m : Calculating the electric field using the
%                            finite difference method.
% Assumption: Propagation in vacuum, no variations in the refractive index,
%             n(x,z) = 1, for all x and z.

% zFieldInit: Vector containing the initial electric field along the z-axis
% zVect: Vector containing the z-coordinates of interest
% xVect: Vector containing the x-coordinates of interest
% frequency: The frequency of operation

function uValues = finiteDifferenceMethod(zFieldInit,zVect,xVect,frequency)
c = 3*10^8;
lambda = c/frequency;
k = 2*pi/lambda;
numZpoints = length(zVect);
numXpoints = length(xVect);

% Finding the ''grid'' of a- and b-values:
bGrid = zeros(numZpoints,numXpoints);
deltaZvect = zeros(numZpoints,1);
deltaZvect(1:numZpoints-1,1)= zVect(2:numZpoints,1)-zVect(1:numZpoints-1);
deltaZvect(numZpoints,1) = deltaZvect(numZpoints-1,1);

deltaXvect = zeros(numXpoints,1);
deltaXvect(1:numXpoints-1,1)= xVect(2:numXpoints,1)-xVect(1:numXpoints-1);
deltaXvect(numXpoints,1) = deltaXvect(numXpoints-1,1);

aGrid = bGrid;
% Filling in the a- and b-grid:
for x = 1: numXpoints
    bGrid(:,x) = 1j.*4.*k.*(deltaZvect.^2)./deltaXvect(x,1);
    aGrid(:,x) = 0;% (k.^2).*(deltaZvect.^2);
end

% Calculate the u-values for successive x-values:
uGrid = zeros(numZpoints,numXpoints);
uGrid(:,1)= zFieldInit;
onesVect = ones(numZpoints-1,1);
VmMat = zeros(numZpoints,numZpoints);
VmRes = zeros(numZpoints,1);
AmMat =  zeros(numZpoints,numZpoints);
for x = 2:numXpoints
    % Filling in the Um and Vm matrices for field propagation estimation:
    diagVmat = 2 + bGrid(:,x)- aGrid(:,x);
    diagVmat(1,1) = 1;
    diagVmat(numZpoints,1) = 1;
    VmMat = diag(diagVmat)+ (-1.*diag(onesVect,-1))+(-1.*diag(onesVect,1)); 
    VmRes = VmMat*uGrid(:,x-1);
    
    diagAmat = -2 + bGrid(:,x)+ aGrid(:,x);
    diagAmat(1,1) = 1;
    diagAmat(numZpoints,1) = 1;
    AmMat = diag(diagAmat)+diag(onesVect,-1) + diag(onesVect,1);
    %AmMat*uGrid(:,x) = VmRes
    uGrid(:,x) = linsolve(AmMat,VmRes);
end

uValues = uGrid;
uValues(abs(uValues)<10^-3) = 10^-3; 
%uValues(abs(uValues)>10^-1) = 10^-1;
end
