% finiteDifferenceMethodIrregularTerrain.m : Calculates the electric field 
%                            using the finite difference method for a given
%                            terrain profile (has to be hump).
% Assumption: Propagation in vacuum, no variations in the refractive index,
%             n(x,z) = 1, for all x and z.

% zFieldInit: Vector containing the initial electric field along the z-axis
% zVect: Vector containing the z-coordinates of interest
% xVect: Vector containing the x-coordinates of interest
% zSurfaceVect: Vector containing the z-coordinates of the surface
% frequency: The frequency of operation

function uValues = finiteDifferenceMethodIrregularTerrain(zFieldInit, ...
    zVect,xVect,zSurfaceVect,frequency)
c = 3*10^8;
lambda = c/frequency;
k = 2*pi/lambda;
numZpoints = length(zVect);
numXpoints = length(xVect);
zSurfaceVect = verticalVector(zSurfaceVect);
xVect = verticalVector(xVect);
zVect= verticalVector(zVect);

% Mapping the surface-vector to correspond with the heights in zVect
% The values of the new surface vector contains the indices corresponding
% to the height in the zVect
zSurfaceIndices = zeros(numXpoints,1);
for x = 1:numXpoints
    zSurfaceIndices(x,1) = find(zVect >=zSurfaceVect(x,1),1,'first');
end


% Finding the ''grid'' of a- and b-values:
deltaZ = zVect(2,1)-zVect(1,1);
deltaX = xVect(2,1)-xVect(1,1);
bGrid = 1j.*ones(numZpoints,1).*4.*k.*(deltaZ.^2)./deltaX;
aGrid = zeros(numZpoints,1);


% Calculate the u-values for successive x-values:
uGrid = zeros(numZpoints,numXpoints);
uGrid(:,1)= zFieldInit;
onesVect = ones(numZpoints-1,1);
VmMat = zeros(numZpoints,numZpoints);
VmRes = zeros(numZpoints,1);
AmMat =  zeros(numZpoints,numZpoints);
for x = 2:numXpoints
    % Filling in the Um and Vm matrices for field propagation estimation:
    diagVmat = 2 + bGrid(:,1)- aGrid(:,1);
    diagVmat(1,1) = 1;
    diagVmat(numZpoints,1) = 1;
    VmMat = diag(diagVmat)+ (-1.*diag(onesVect,-1))+(-1.*diag(onesVect,1)); 
    VmRes = VmMat*uGrid(:,x-1);
    
    diagAmat = -2 + bGrid(:,1)+ aGrid(:,1);
    diagAmat(1,1) = 1;
    diagAmat(numZpoints,1) = 1;
    AmMat = diag(diagAmat)+diag(onesVect,-1) + diag(onesVect,1);
    %AmMat*uGrid(:,x) = VmRes
    uGrid(:,x) = linsolve(AmMat,VmRes);
    
    if zSurfaceIndices(x,1) > 1
        uGrid(1:(zSurfaceIndices(x,1)-1),x) = 0;
    end
        
end

uValues = uGrid;
uValues(abs(uValues)<10^-3) = 10^-3; 
%uValues(abs(uValues)>10^-1) = 10^-1;
end
