% getDiffractionCoeff.m: Returns the diffraction coefficient for a wedge
%                        for a given set of parameters.


function D = getUTDfield(Xn,Zn,deltaX,frequency)
c=3.*10^8;
lambda = c./frequency;
k = 2.*pi/lambda;
[xVect,zVect] = interpolate(Xn,Zn, frequency,'linear', deltaX);
xVect = verticalVector(xVect);
numXpoints = length(xVect);

tx_x = xVect(1,1);
rx_x = xVect(numXpoints,1);
tx_z = tx;
rx_z = rx;

% The an, bn, cn, and dn coefficients, ITU-R report p.4
an_coeffs = [+1.595769140, -0.000001702, -6.808568854, -0.000576361, ...
    +6.920691902, -0.016898657, -3.050485660, -0.075752419,+0.850663781 ...
    -0.025639041, -0.150230960, +0.034404779];

bn_coeffs = [-0.000000033, +4.255387524, -0.000092810, -7.780020400, ...
    -0.009520895, +5.075161298, -0.138341947, -1.363729124,-0.403349276 ...
    +0.702222016, -0.216195929, +0.019547031]; 

cn_coeffs = [+0.000000000, -0.024933975, +0.000003936, +0.005770956 ...
    +0.000689892, -0.009497136, +0.011948809, -0.006748873,+0.000246420 ...
    +0.002102967, -0.001217930, +0.000233939];

dn_coeffs = [+0.199471140, +0.000000023, -0.009351341, +0.000023006, ...
    +0.004851466, +0.001903218, -0.017122914, +0.029064067,-0.027928955 ...
    +0.016497308, -0.005598515, +0.000838386];

% Finding the parameters:

%s1:
index = find(zVect == max(zVect(:)));

s1 = sqrt( ((xVect(index)-tx_x).^2) + ((zVect(index)-tx_z).^2));

%s2:
s2 = sqrt( ((rx_x-xVect(index)).^2) + ((zVect(index)-rx_z).^2));

% The diffraction coefficient: 

% Incidence angle (phi1), measured from fase 0:
theta1aux = atan(zVect(index)/(xVect(index)-tx_x));
theta1 = (pi/2) - theta1aux;
phi1 = asin(sin(theta1)*tx_z/s1);

% Diffraction angle (phi2), measured from fase 0:
alpha1aux = atan((xVect(index)-tx_x)/zVect(index));
alpha2aux = atan((rx_x-xVect(index))/zVect(index));
theta2aux = atan(zVect(index)/(rx_x-xVect(index)));
theta2 = (pi/2) - theta2aux;
alpha3aux = asin(sin(theta2)*zvect(index)/s2);
phi2 = (2*pi) - alpha1aux - alpha2aux - alpha3aux;

% External wedge angle:
phiWedge = (2.*pi) - alpha1aux - alpha2aux;
n = phiWedge./pi;

% The other coefficients:
L = (s1.*s2./(s1+s2));

betaPlus = phi2 + phi1;
betaMinus = phi2 - phi1;

Nplus = round((betaPlus + pi)./(2.*n.*pi));
Nminus = round((betaMinus - pi)./(2.*n.*pi));

aPlus = 2.*cos(((2.*n.*pi.*Nplus)-betaPlus)./2);
aMinus =  2*cos(((2.*n.*pi.*Nminus)-betaMinus)./2);

% Horizontal polarization with respect to the ground. The ground is almost
% horizonal due to small hegiht differences.
R = -1; % reflection coefficient. For both sides of the wedge. ? TODO!!

% The calculations:
arg1 = k.*L.*aPlus*betaMinus;
switch arg1
    case arg1 < 4
        coeff_sum = sum( (an_coeffs + 1j.*bn_coeffs).*((arg1./4).^n));
        A_arg1 = ((1-1j)./2) - exp(-1j.*arg1).*sqrt(arg1./4)*coeff_sum;
    otherwise
        coeff_sum = sum( (cn_coeffs + 1j.*dn_coeffs).*((4./arg1).^n));
        A_arg1 = -exp(-1j.*arg1).*sqrt(4./arg1).*coeff_sum;
end


F_arg1 = 1j.*2.*sqrt(arg1).*exp(1j.*x).*((sqrt(pi./8)-(1-1j)) - ...
    (sqrt(pi./2).*A_arg1));
coeff1 = cot((pi+betaMinus)./(2.*n)).*F_arg1;

arg2 = k.*L.*aMinus.*betaMinus;
switch arg2
    case arg2 < 4
        coeff_sum = sum( (an_coeffs + 1j.*bn_coeffs).*((arg2./4).^n));
        A_arg2 = ((1-1j)./2) - exp(-1j.*arg2).*sqrt(arg2./4).*coeff_sum;
    otherwise
        coeff_sum = sum( (cn_coeffs + 1j.*dn_coeffs).*((4./arg1).^n));
        A_arg2 = -exp(-1j.*arg2).*sqrt(4./arg2).*coeff_sum;
end

F_arg2 = 1j.*2.*sqrt(arg2).*exp(1j.*x).*((sqrt(pi./8)-(1-1j)) - ...
    (sqrt(pi/2)*A_arg2));
coeff2 = cot((pi-betaMinus)./(2.*n)).*F_arg2;

arg3 = k.*L.*aMinus.*betaPlus;
switch arg3
    case arg3 < 4
        coeff_sum = sum( (an_coeffs + 1j.*bn_coeffs).*((arg3./4).^n));
        A_arg3 = ((1-1j)./2) - exp(-1j.*arg3).*sqrt(arg3./4).*coeff_sum;
    otherwise
        coeff_sum = sum( (cn_coeffs + 1j.*dn_coeffs).*((4./arg3).^n));
        A_arg3 = -exp(-1j.*arg3).*sqrt(4./arg3).*coeff_sum;
end

F_arg3 = 1j.*2.*sqrt(arg3).*exp(1j.*x).*((sqrt(pi./8)-(1-1j)) - ...
    (sqrt(pi./2).*A_arg3));
coeff3 = cot((pi-betaPlus)./(2.*n)).*F_arg3;

arg4 = k.*L.*aPlus.*betaPlus;
switch arg4
    case arg4 < 4
        coeff_sum = sum( (an_coeffs + 1j.*bn_coeffs).*((arg4./4).^n));
        A_arg4 = ((1-1j)./2) - exp(-1j.*arg4).*sqrt(arg4./4).*coeff_sum;
    otherwise
        coeff_sum = sum( (cn_coeffs + 1j.*dn_coeffs).*((4./arg4).^n));
        A_arg4 = -exp(-1j.*arg4).*sqrt(4./arg4).*coeff_sum;
end

F_arg4 = 1j.*2.*sqrt(arg4).*exp(1j.*x)*((sqrt(pi./8)-(1-1j)) - ...
    (sqrt(pi./2).*A_arg4));
coeff4 = cot((pi+betaPlus)/(2.*n)).*F_arg4;


Daux = coeff1 + coeff2 + R.*coeff3 + R.*coeff4;

D = Daux.*(-exp(-1j.*pi./4)./(2.*n.*sqrt(2.*pi.*k))); % The diffraction coefficient

end