% impedanceBoundary_SSA.m:

function [uValVect] = impedanceBoundary_SSA(uIn,deltaX,deltaZ,frequency,alpha)

% Implementation of section 9.4 in Levy.
uIn = verticalVector(uIn);
numPoints = length(uIn);
c = 3*10^8;
lambda = c/frequency;
k = 2*pi/lambda;


r = -sqrt(1+((alpha*deltaZ).^2)) - (alpha.*deltaZ); % Horizontal polarization
abs(r)
A = (2*(1-(r^2)))/((1+(r^2))*(1-(r^(2*(numPoints-1)))));

% The procedure:

% 1) Compute the matched function:
gVect = discreteMatchedFunction(uIn,deltaZ,alpha);

% 2) Compute the discrete sine transform of gVect:
Fvect = zeros(numPoints,1);

% The first coefficient F0, and the last coeff FN:
Fvect(1,1) = Fvect(1,1) + 0.5*uIn(1,1);
Fvect(numPoints,1) =Fvect(numPoints,1) + 0.5*((-r)^(numPoints-1))*uIn(1,1);
for m = 2:(numPoints-1)
    Fvect(1,1) = Fvect(1,1)+ r^(m-1)*uIn(m,1);
    Fvect(numPoints,1) = Fvect(numPoints,1) + ((-r)^(numPoints-1-m))*uIn(m,1);
end
Fvect(1,1) = A.*(Fvect(1,1) +  0.5*r^(numPoints-1)*uIn((numPoints-1),1));
Fvect(numPoints,1) =A.*(Fvect(numPoints,1) + 0.5*uIn(1,1));

% Coefficient m = 1:N-1 :
indiceVect = verticalVector(1:1:(numPoints-2));
for m = 2:(numPoints-1)
    Fvect(2:(numPoints-1),1) =  Fvect(2:(numPoints-1),1) + ...
        gVect((m-1),1).*sin(pi.*indiceVect.*(m-1)./(numPoints-1));
end

% 3) Advancing in range:

Fadv = zeros(numPoints,1);

% F0
Fadv(1,1) = exp((1j*deltaX/(2*k*(deltaZ^2)))*((r-1)^2)/r)*Fvect(1,1);

% Coefficient m = 1:N-1
Fadv(2:(numPoints-1),1) = exp((1j.*deltaX./(k.*(deltaZ.^2)))...
    .*cos((pi.*indiceVect./(numPoints-1))-1)).*Fvect(2:(numPoints-1),1);

% FN
Fadv(numPoints,1) = exp((-1j*deltaX/(2*k*(deltaZ^2)))*((r+1)^2)/r)...
    *Fvect(numPoints,1);

% 4) Compute the inverse discrete sine transform
% gVect2 = zeros(numPoints,1);
% gVect2(2:(numPoints-1),1) = inverseDiscreteSineTrans(Fadv(2:(numPoints-1));

% 5) Computing directly the advanced f coeff:

fVect2 = zeros(numPoints,1);
for x=1:numPoints
    m = x-1;
    betaArr = sin(pi.*indiceVect.*m./(numPoints-1))./deltaZ;
    betaCosCoeff = betaArr.*cos(pi.*indiceVect.*m./(numPoints-1));
    
    alphaSinCoeff = alpha.*sin(pi.*indiceVect.*m./(numPoints-1));
    
    fVect2(x,1) = ((r^m)*Fadv(1,1)) +  ...
        (2/(numPoints-1))*sum( ((alphaSinCoeff - betaCosCoeff)./ ...
        (alpha.^2 + betaArr.^2)).*Fadv(2:(numPoints-1),1)) + ...
        ((-r).^((numPoints-1)-m)).*Fadv(numPoints,1);
end
uValVect = fVect2;
end













