% interpolate : Interpolates the given points according to the frequency so
% that the sampling distance between the points in the x-direction is less
% than wavelength/2.

% Xn: Vector of x-values
% Zn: Vector of corresponding z-values
% frequency: The frequency of operation [Hz]
% mode: 'linear' or 'curve' interpolation
% deltaX: spacing between the sampling points along the x-axis, if set to 0
%         the spacing is according to Nyquist's theorem.

% Returns: X- and Z-vectors containg the interpolated values.

% (tested)

function[Xni,Zni] = interpolate(Xn,Zn, frequency,mode, deltaX)
c = 3*10^8; % Speed of light
lambda = c/frequency; % Wavelength

switch deltaX
    case 0
        maxSampleDist = lambda/2;
        sampleDist = maxSampleDist - 0.1*maxSampleDist;
    otherwise
        sampleDist = deltaX;
end

% Creating array containing x-values with appropriate sampling distance:
Xni = [Xn(1):deltaX:Xn(length(Xn))];

% Interpolating the given values: 
switch mode
    case 'linear'
        Zni = interp1(Xn,Zn,Xni);
    case 'curve'
        Zni = spline(Xn,Zn,Xni);
end
end
    



