% pathLossFlat.m: Calculates the path loss at a flat surface, used for 
%                 comparing with plane earth loss model, assuming zero
%                 reflection loss.
% tx: Transimtter antenna height
% rx: Receiver antenna height
% distance: The distance between the antennas, ignoring height difference
% eField: Calculated electric field along the surface at a given height,
%         zs, may contain multiple heights of consideration
% zs: The height eField is taken from
% frequency: The frequency of operation
% simulationType: Cell-array containing strings with the name of the
%                 simulation types used.


function pathLossFlat(tx,rx,xVect,eField,zs, frequency,simulationType, plotTitle)
c = 3*10^8;
lambda = c/frequency;
k = 2*pi/lambda;
numXpoints = length(xVect);
xVect = verticalVector(xVect);

% Finding the dimensions of the incoming e-field array:
[lines,columns] = size(eField);
if(lines>columns)
    eField = eField';
    numCases = columns;
else
    numCases = lines;
end
cmap = hsv(numCases+3);

plotNames = cell(1+length(simulationType),1);
plotNames{1,1} = 'Plane Earth Loss';
for i = 1: length(simulationType);
    plotNames{i+1,1} = simulationType{i};
end
simulationType = verticalVector(simulationType);
simulationType = vertcat('Plane Earth Loss 40 dB', simulationType);
simulationType = vertcat('Plane Earth Loss Wrong', simulationType);
plotNames = vertcat('Plane Earth Loss', simulationType);


% Calculating the plane earth loss (path loss for plane earth):
figure()
Pl =  (lambda./(4.*pi.*xVect)).*(abs(1 +exp(1j.*k.*2.*tx.*rx./xVect)).^2);
Pl2_feil_kanskje =  (lambda./(4.*pi.*xVect)).*(abs(1 +exp(1j.*k.*2.*tx.*rx./xVect)));
Pl_dB = 20.*log10(verticalVector(Pl(10:numXpoints,1)));
Pl_dB2 = 20.*log10(verticalVector(Pl2_feil_kanskje(10:numXpoints,1)));

semilogx(xVect(10:numXpoints,1),Pl_dB,'Color',cmap(1,:));
hold on
semilogx(xVect(10:numXpoints,1),Pl_dB2,'Color',cmap(2,:));
xlabel('r[m]');
ylabel('Path Loss [-dB]'); 
%legend('Plane Earth Loss');

% Plane earth loss at where it starts to increase [dB]:
L_PEL = -(40*log10(xVect) -20*log10(tx)-20*log10(rx));
hold on
semilogx(xVect(10:numXpoints,1),L_PEL(10:numXpoints,1),'Color',cmap(3,:));

for i = 1: numCases
    hold on
    
    % Newly inserted:
    pl1 = verticalVector(20.*log10(abs(eField(i,:))));
    pl2 = - 10.*log10(xVect);
    %pl3 = pl1 + pl2;
    pathLoss = pl1 + pl2; %20.*log10(abs(eField(i,:))) - 10.*log10(xVect); % - 20.*log10(4.*pi) + 30.*log(lambda);
    semilogx(xVect(10:numXpoints,1),pathLoss(10:numXpoints,1),'Color',cmap(i+3,:)); %,'LineWidth',1*(numCases+1-i));
    % Calculating the path loss for the incoming eField
    %semilogx(xVect,20.*log10(abs(eField(i,:))),colors(i));
    %legend(simulationType(i));
end
legend(plotNames);
title(plotTitle);
grid on



end



