% pathLossIndra_alongX.m: Compare the relative field strengths from the
%                            simulated results with the results from a
%                            given file. Horizontal comparison.

% rx_xArr: Receiver antenna (obseravation points) coordinates along the
%          x-axis.
% rx_zArr: Receiver antenna (observation points) coordinates along the
%          z-axis.
% eField: Calculated electric field at the coordinates to rx_xArr and
%         rx_zArr.
% frequency: The frequency of operation
% simulationType: Cell-array containing strings with the name of the
%                 simulation types used.
% plotTitle: Title of the plot.
% saveFig: file name of the plot, leave blank (' ') if the plot should not
%          be saved.
% compareFile: File name and relative path if necessary of the file to
%              compare results with. Leave blank (' ') if there is no file
%              to compare with.


function pathLossIndra_alongX(rx_xArr,rx_zArr,eField,...
    frequency,simulationType, plotTitle,saveFig,compareFile)
c = 3*10^8;
lambda = c/frequency;
k = 2*pi/lambda;
columnNumber = 2;
if compareFile ~= ' '
    
    [eFieldValues,zVectIndra]=importFieldResultsFromFile(...
        compareFile,columnNumber);
end

rx_zArr = verticalVector(rx_zArr);
numRxPoints = length(rx_zArr);

thisFontSize = 10;


% Finding the dimensions of the incoming e-field array:
[lines,columns] = size(eField);
if(lines>columns)
    eField = eField';
    numCases = columns;
else
    numCases = lines;
end
cmap = hsv(numCases+1);

if compareFile ~= ' '
    plotNames = cell(1+length(simulationType),1);
    plotNames{1,1} = 'E-field Indra';
    for i = 1: length(simulationType);
        plotNames{i+1,1} = simulationType{i};
    end
else % No file to compare with
    plotNames =simulationType;
end

% Save the figure?
saveFigVal = isempty(saveFig);

switch saveFigVal
    case 1
        % Not save figure
        %figure()
    case 0
        % Save figure
        fig = figure('visible','on');
end
lineWidth  = 1;
if compareFile ~= ' '
 Pl =eFieldValues; % In dB already
 Pl_dB =Pl;
 semilogx(zVectIndra,Pl_dB,'Color',cmap(1,:),'LineWidth',lineWidth);
 minCompareValue = min(Pl_dB);
end
ax1 = gca;
set(ax1,'XColor','k','YColor','k')
%legend(plotNames{1,1})
set(ax1, 'Box', 'off')
set(ax1, 'Color', 'none')
 for i = 1:numCases
     if compareFile ~= ' '
         hold on
     elseif i > 1
         hold on
     end
    pl1 = smooth(verticalVector(20.*log10(abs(eField(i,:)))));
    pl2 = 0; % - 10.*log10(xVect);
    minValueField = min(pl1);
   
    pathLoss = pl1 + pl2; 
    
    
   semilogx(rx_xArr,pathLoss,'Color',cmap(5-i+1,:),'LineWidth',lineWidth);
   
     
 end
 grid on
set(gca,'FontSize',thisFontSize) 
legend(plotNames,'Location','NorthEast')
ylabel('Relative field strength [dB]');
xlabel('Distance [m]');

switch saveFigVal
    case 0
        % Save figure
        titleFig = verticalVector(saveFig)';
        regexprep(titleFig,' ','_');
        regexprep(titleFig,'\','_');
        regexprep(titleFig,':','_');
        regexprep(titleFig,'=','_');
        
        if (isempty(strfind(titleFig,'.png')) == 1)
            % File extension needs to be added.
            titleFig = horzcat(titleFig,'.png');
        end
        
        saveas(fig,titleFig,'png');
        titleFig = titleFig(1:(length(titleFig)-4));
        titleFig = horzcat(titleFig,'.pdf');
        save2pdf(titleFig);
end

end