% splitStepAlgorithm.m: Calculates the electric field using the split-step
%                       algorithm
% Assumption: - Propagation in vacuum, no variations in the refractive index,
%               n(x,z) = 1, for all x and z.
% Remarks: length(xVect) >= length(zVect)             

% zFieldInit: Vector containing the initial electric field along the z-axis
% zVect: Vector containing the z-coordinates of interest
% xVect: Vector containing the x-coordinates of interest
% frequency: The frequency of operation


function uValues = splitStepAlgorithm(zFieldInit,zVect,xVect,frequency)

c = 3*10^8;
lambda = c/frequency;
k = 2*pi/lambda;
numZpoints = length(zVect);
numXpoints = length(xVect);

deltaZvect = zeros(numZpoints,1);
deltaZvect(1:numZpoints-1,1)= zVect(2:numZpoints,1)-zVect(1:numZpoints-1);
deltaZvect(numZpoints,1) = deltaZvect(numZpoints-1,1);

deltaXvect = zeros(numXpoints,1);
deltaXvect(1:numXpoints-1,1)= xVect(2:numXpoints,1)-xVect(1:numXpoints-1);
deltaXvect(numXpoints,1) = deltaXvect(numXpoints-1,1);

uGrid = zeros(numZpoints,numXpoints);
uGrid(:,1)= zFieldInit;
indicesZminusOne = verticalVector(linspace(1,(numZpoints-1),(numZpoints-1)));
Pprime = exp(-1j.*(pi.^2).*(indicesZminusOne.^2)...
    .*deltaXvect(1:1,1)./(2.*k.*(numZpoints.^2)));

% The Pprime expression below can be deleted:
%Pprime = exp(-1j.*(pi.^2).*(zVect(1:(numZpoints-1),1).^2)...
%    .*deltaXvect(1:(numZpoints-1),1)./(2.*k.*(numZpoints.^2)));

Pprime = verticalVector(Pprime);
%Pprime = vertcat(Pprime,0);

for x = 2:numXpoints
    uGrid(1:(numZpoints-1),x) = inverseDiscreteSineTrans( ...
        Pprime.*discreteSineTrans(uGrid(1:(numZpoints-1),x-1)));
end
uValues = uGrid;
uValues(abs(uValues)<10^-3) = 10^-3; 
end

