% splitStepAlgorithmIrregularTerrain.m: Uses the split-step-algoritm to 
%                                       calculate the wave propagation over
%                                       irregular terrain.

% zFieldInit: Vector containing the initial electric field along the z-axis
% zVect: Vector containing the z-coordinates of interest
% xVect: Vector containing the x-coordinates of interest
% zSurfaceVect: Vector containing the z-coordinates of the surface
% frequency: The frequency of operation
function [uValues] =splitStepAlgorithmIrregularTerrain(zFieldInit,zVect,...
    xVect,zSurfaceVect,frequency)

c = 3*10^8;
lambda = c/frequency;
k = 2*pi/lambda;
numZpoints = length(zVect);
numZpointsInit = numZpoints;
numXpoints = length(xVect);
zVect = verticalVector(zVect);
xVect = verticalVector(xVect);
zSurfaceVect = verticalVector(zSurfaceVect);

deltaZ = zVect(2,1) - zVect(1,1);
deltaX = xVect(2,1) - xVect(1,1);

% Finding the maximum height of the surface:
maxZ = max(zSurfaceVect);

% The number of points to add:
addVect =verticalVector([zVect(numZpoints,1)+deltaZ:deltaZ:...
    (xVect(numZpoints,1)+ maxZ+deltaZ)]);
zVect = vertcat(zVect,addVect);
numZpoints = length(zVect);
% Mapping the surface-vector to correspond with the heights in zVect
% The values of the new surface vector contains the indices corresponding
% to the height in the zVect
zSurfaceIndices = zeros(numXpoints,1);
for x = 1:numXpoints
    zSurfaceIndices(x,1) = find(zVect >=zSurfaceVect(x,1),1,'first');
end



uGrid = zeros(numZpoints,numXpoints);
uGrid(1:numZpointsInit,1)= zFieldInit;
indicesZminusOne = verticalVector(linspace(1,(numZpointsInit-1), ...
    (numZpointsInit-1)));
Pprime = exp(-1j.*(pi.^2).*(indicesZminusOne.^2)...
    .*deltaX./(2.*k.*(numZpointsInit.^2)));
Pprime = verticalVector(Pprime);

% Creating the vGrid:
vGrid = zeros(numZpointsInit,numXpoints);

% Creating the vector containing the slopes of the terrain
alphaVect =  zSurfaceVect(2:numXpoints,1)-zSurfaceVect(1:(numXpoints-1),1);
alphaVect = vertcat(0,alphaVect);

vGrid(:,1) = uGrid(zSurfaceIndices(1):(zSurfaceIndices(1)+numZpointsInit-1),1);
for x = 2:numXpoints 
    vGrid(:,x) = uGrid(zSurfaceIndices(x-1):zSurfaceIndices(x-1)...
        +numZpointsInit-1,x-1).*exp(1j.*k.*(alphaVect(x,1)-alphaVect(x-1,1))...
        .*zVect(1:numZpointsInit,1));
    vGrid(1:(numZpointsInit-1),x) = inverseDiscreteSineTrans( ...
        Pprime.*discreteSineTrans(vGrid(1:(numZpointsInit-1),x)));
    uGrid(zSurfaceIndices(x,1):zSurfaceIndices(x)+numZpointsInit-1,x) ...
        = vGrid(:,x).*exp(1j.*k.*alphaVect(x).*zVect(1:numZpointsInit,1));
end
uValues = uGrid;
uValues(abs(uValues)<10^-3) = 10^-3; 
end
