% subDiagonalElimination.m: Performs sub-diagonal elimination of a square 
%                           tridiagonal matrix in a system with N unknowns
%                           triDiagMat*U = resultIn, U is unknown 
% triDiagMat: square tri-diagonal matrix
% return: subDiagonalEli: the sub-diagonal of the tri-diagonal matrix
%                         eliminated. 
%                         Resulting in: subDiagonalEli*U = resultOut                         

function [subDiagonalEli,resultOut] = subDiagonalElimination(triDiagMat,resultIn)

% numElts = length = diagElts;
% subDiagElts = ones(numElts-1,1);
% subDiagElts(numElts-1,1) = 0;
% 
% supDiagElts = ones(numElts-1,1);
% supDiagElts(1,1) = 0;

numElts = length(resultIn);
resultOut = zeros(length(resultIn),1);
resultIn = verticalVector(resultIn);

subDiagonalEli = zeros(size(triDiagMat));

% First row:
tau0 = resultIn(1)/triDiagMat(1,1);
resultOut(1,1) = tau0;
subDiagonalEli(1,1) = triDiagMat(1,1)/ triDiagMat(1,1);


% The remaining rows:
%tauJ = tau0;
%alphaProd = triDiagMat(1,1);
for j = 2:numElts-1;
    subDiagonalEli(j,:) = (triDiagMat(j,:).*subDiagonalEli(j-1,j-1) - subDiagonalEli(j-1,:));
    subDiagonalEli(j,:) = subDiagonalEli(j,:)./subDiagonalEli(j,j+1);
     resultOut(j,1) = (resultIn(j,1)*subDiagonalEli(j-1,j-1)-resultOut(j-1,1))./subDiagonalEli(j,j+1);

end
subDiagonalEli(numElts,numElts) = triDiagMat(numElts,numElts);
resultOut(numElts,1) = resultIn(numElts,1);
end

