% supDiagonalElimination.m: Performs sup-diagonal elimination of a square 
%                           tridiagonal matrix in a system with N unknowns
%                           triDiagMat*U = resultIn, U is unknown
% triDiagMat: square tri-diagonal matrix. May or may not contain any
%             elements at the sub-diagonal.
% return: supDiagonalEli: the sub-diagonal of the tri-diagonal matrix
%                         eliminated. 
%                         Resulting in: supDiagonalEli*U = resultOut                         



function [supDiagonalEli,resultOut] = supDiagonalElimination(triDiagMat,resultIn)

numElts = length(resultIn);
resultOut = zeros(length(resultIn),1);
resultIn = verticalVector(resultIn);

supDiagonalEli = zeros(size(triDiagMat));

% The last row:
tau0 = resultIn(numElts)/triDiagMat(numElts,numElts);
resultOut(numElts,1) = tau0;
supDiagonalEli(numElts,numElts) = triDiagMat(numElts,numElts)/ triDiagMat(numElts,numElts);


% The remaining rows:
%tauJ = tau0;
%alphaProd = triDiagMat(1,1);
for j = numElts-1:-1:2;
   
    supDiagonalEli(j,:) = (triDiagMat(j,:).*supDiagonalEli(j+1,j+1) - supDiagonalEli(j+1,:));
    supDiagonalEli(j,:) =supDiagonalEli(j,:)./(supDiagonalEli(j,j));
     resultOut(j,1) = ((resultIn(j,1)*supDiagonalEli(j+1,j+1)) ...
        - resultOut(j+1,1))./supDiagonalEli(j,j);
end
supDiagonalEli(1,1) = triDiagMat(1,1);
resultOut(1,1) = resultIn(1,1);
end